@@ -17,13 +17,13 @@ use std::error::Error as _;
 use std::io;
 use std::io::Write as _;
 use std::iter;
-use std::str;
 use std::sync::Arc;
 
 use itertools::Itertools as _;
 use jj_lib::absorb::AbsorbError;
 use jj_lib::backend::BackendError;
 use jj_lib::backend::CommitId;
+use jj_lib::bisect::BisectionError;
 use jj_lib::config::ConfigFileSaveError;
 use jj_lib::config::ConfigGetError;
 use jj_lib::config::ConfigLoadError;
@@ -69,6 +69,7 @@ use crate::description_util::TextEditError;
 use crate::diff_util::DiffRenderError;
 use crate::formatter::FormatRecorder;
 use crate::formatter::Formatter;
+use crate::formatter::FormatterExt as _;
 use crate::merge_tools::ConflictResolveError;
 use crate::merge_tools::DiffEditError;
 use crate::merge_tools::MergeToolConfigError;
@@ -559,13 +560,13 @@ jj currently does not support partial clones. To use jj with this repository, tr
 
     impl From<GitFetchError> for CommandError {
         fn from(err: GitFetchError) -> Self {
-            if let GitFetchError::InvalidBranchPattern(pattern) = &err {
-                if pattern.as_exact().is_some_and(|s| s.contains('*')) {
-                    return user_error_with_hint(
-                        "Branch names may not include `*`.",
-                        "Prefix the pattern with `glob:` to expand `*` as a glob",
-                    );
-                }
+            if let GitFetchError::InvalidBranchPattern(pattern) = &err
+                && pattern.as_exact().is_some_and(|s| s.contains('*'))
+            {
+                return user_error_with_hint(
+                    "Branch names may not include `*`.",
+                    "Prefix the pattern with `glob:` to expand `*` as a glob",
+                );
             }
             match err {
                 GitFetchError::NoSuchRemote(_) => user_error(err),
@@ -743,6 +744,14 @@ impl From<FixError> for CommandError {
     }
 }
 
+impl From<BisectionError> for CommandError {
+    fn from(err: BisectionError) -> Self {
+        match err {
+            BisectionError::RevsetEvaluationError(_) => user_error(err),
+        }
+    }
+}
+
 fn find_source_parse_error_hint(err: &dyn error::Error) -> Option<String> {
     let source = err.source()?;
     if let Some(source) = source.downcast_ref() {
@@ -1008,41 +1017,37 @@ pub fn print_error_sources(ui: &Ui, source: Option<&dyn error::Error>) -> io::Re
     let Some(err) = source else {
         return Ok(());
     };
-    ui.stderr_formatter()
-        .with_label("error_source", |formatter| {
-            if err.source().is_none() {
-                write!(formatter.labeled("heading"), "Caused by: ")?;
-                writeln!(formatter, "{err}")?;
-            } else {
-                writeln!(formatter.labeled("heading"), "Caused by:")?;
-                for (i, err) in iter::successors(Some(err), |&err| err.source()).enumerate() {
-                    write!(formatter.labeled("heading"), "{}: ", i + 1)?;
-                    writeln!(formatter, "{err}")?;
-                }
-            }
-            Ok(())
-        })
+    let mut formatter = ui.stderr_formatter().into_labeled("error_source");
+    if err.source().is_none() {
+        write!(formatter.labeled("heading"), "Caused by: ")?;
+        writeln!(formatter, "{err}")?;
+    } else {
+        writeln!(formatter.labeled("heading"), "Caused by:")?;
+        for (i, err) in iter::successors(Some(err), |&err| err.source()).enumerate() {
+            write!(formatter.labeled("heading"), "{}: ", i + 1)?;
+            writeln!(formatter, "{err}")?;
+        }
+    }
+    Ok(())
 }
 
 fn print_error_hints(ui: &Ui, hints: &[ErrorHint]) -> io::Result<()> {
+    let mut formatter = ui.stderr_formatter().into_labeled("hint");
     for hint in hints {
-        ui.stderr_formatter().with_label("hint", |formatter| {
-            write!(formatter.labeled("heading"), "Hint: ")?;
-            match hint {
-                ErrorHint::PlainText(message) => {
-                    writeln!(formatter, "{message}")?;
-                }
-                ErrorHint::Formatted(recorded) => {
-                    recorded.replay(formatter)?;
-                    // Formatted hint is usually multi-line text, and it's
-                    // convenient if trailing "\n" doesn't have to be omitted.
-                    if !recorded.data().ends_with(b"\n") {
-                        writeln!(formatter)?;
-                    }
+        write!(formatter.labeled("heading"), "Hint: ")?;
+        match hint {
+            ErrorHint::PlainText(message) => {
+                writeln!(formatter, "{message}")?;
+            }
+            ErrorHint::Formatted(recorded) => {
+                recorded.replay(formatter.as_mut())?;
+                // Formatted hint is usually multi-line text, and it's
+                // convenient if trailing "\n" doesn't have to be omitted.
+                if !recorded.data().ends_with(b"\n") {
+                    writeln!(formatter)?;
                 }
             }
-            io::Result::Ok(())
-        })?;
+        }
     }
     Ok(())
 }
