@@ -30,20 +30,15 @@ use crate::ui::Ui;
 /// Create a new bookmark
 #[derive(clap::Args, Clone, Debug)]
 pub struct BookmarkCreateArgs {
-    // TODO(#5374): Make required in jj 0.32+
     /// The bookmark's target revision
-    //
-    // The `--to` alias exists for making it easier for the user to switch
-    // between `bookmark create`, `bookmark move`, and `bookmark set`. Currently target revision
-    // defaults to the working copy if not specified, but in the near future it will be required to
-    // explicitly specify it.
     #[arg(
         long, short,
+        default_value = "@",
         visible_alias = "to",
         value_name = "REVSET",
         add = ArgValueCompleter::new(complete::revset_expression_all),
     )]
-    revision: Option<RevisionArg>,
+    revision: RevisionArg,
 
     /// The bookmarks to create
     #[arg(required = true, value_parser = revset_util::parse_bookmark_name)]
@@ -56,16 +51,9 @@ pub fn cmd_bookmark_create(
     args: &BookmarkCreateArgs,
 ) -> Result<(), CommandError> {
     let mut workspace_command = command.workspace_helper(ui)?;
-    if args.revision.is_none() {
-        writeln!(
-            ui.warning_default(),
-            "Target revision was not specified, defaulting to the working copy (-r@). In the near \
-             future it will be required to explicitly specify target revision."
-        )?;
-    }
-    let target_commit = workspace_command
-        .resolve_single_rev(ui, args.revision.as_ref().unwrap_or(&RevisionArg::AT))?;
-    let view = workspace_command.repo().view();
+    let target_commit = workspace_command.resolve_single_rev(ui, &args.revision)?;
+    let repo = workspace_command.repo().as_ref();
+    let view = repo.view();
     let bookmark_names = &args.names;
     for name in bookmark_names {
         if view.get_local_bookmark(name).is_present() {
@@ -88,6 +76,9 @@ pub fn cmd_bookmark_create(
             ));
         }
     }
+    if target_commit.is_discardable(repo)? {
+        writeln!(ui.warning_default(), "Target revision is empty.")?;
+    }
 
     let mut tx = workspace_command.start_transaction();
     for name in bookmark_names {
