@@ -32,18 +32,15 @@ use crate::ui::Ui;
 /// Create or update a bookmark to point to a certain commit
 #[derive(clap::Args, Clone, Debug)]
 pub struct BookmarkSetArgs {
-    // TODO(#5374): Make required in jj 0.32+
     /// The bookmark's target revision
-    //
-    // Currently target revision defaults to the working copy if not specified, but in the near
-    // future it will be required to explicitly specify it.
     #[arg(
         long, short,
+        default_value = "@",
         visible_alias = "to",
         value_name = "REVSET",
         add = ArgValueCompleter::new(complete::revset_expression_all),
     )]
-    revision: Option<RevisionArg>,
+    revision: RevisionArg,
 
     /// Allow moving the bookmark backwards or sideways
     #[arg(long, short = 'B')]
@@ -64,15 +61,7 @@ pub fn cmd_bookmark_set(
     args: &BookmarkSetArgs,
 ) -> Result<(), CommandError> {
     let mut workspace_command = command.workspace_helper(ui)?;
-    if args.revision.is_none() {
-        writeln!(
-            ui.warning_default(),
-            "Target revision was not specified, defaulting to the working copy (--revision=@). In \
-             the near future it will be required to explicitly specify target revision."
-        )?;
-    }
-    let target_commit = workspace_command
-        .resolve_single_rev(ui, args.revision.as_ref().unwrap_or(&RevisionArg::AT))?;
+    let target_commit = workspace_command.resolve_single_rev(ui, &args.revision)?;
     let repo = workspace_command.repo().as_ref();
     let bookmark_names = &args.names;
     let mut new_bookmark_count = 0;
@@ -96,6 +85,9 @@ pub fn cmd_bookmark_set(
             ));
         }
     }
+    if target_commit.is_discardable(repo)? {
+        writeln!(ui.warning_default(), "Target revision is empty.")?;
+    }
 
     let mut tx = workspace_command.start_transaction();
     for bookmark_name in bookmark_names {
@@ -120,9 +112,6 @@ pub fn cmd_bookmark_set(
             writeln!(formatter)?;
         }
     }
-    if bookmark_names.len() > 1 && args.revision.is_none() {
-        writeln!(ui.hint_default(), "Use -r to specify the target revision.")?;
-    }
 
     tx.finish(
         ui,
