@@ -26,21 +26,21 @@ use jj_lib::config::ConfigFile;
 use jj_lib::config::ConfigSource;
 use tracing::instrument;
 
-use self::edit::cmd_config_edit;
 use self::edit::ConfigEditArgs;
-use self::get::cmd_config_get;
+use self::edit::cmd_config_edit;
 use self::get::ConfigGetArgs;
-use self::list::cmd_config_list;
+use self::get::cmd_config_get;
 use self::list::ConfigListArgs;
-use self::path::cmd_config_path;
+use self::list::cmd_config_list;
 use self::path::ConfigPathArgs;
-use self::set::cmd_config_set;
+use self::path::cmd_config_path;
 use self::set::ConfigSetArgs;
-use self::unset::cmd_config_unset;
+use self::set::cmd_config_set;
 use self::unset::ConfigUnsetArgs;
+use self::unset::cmd_config_unset;
 use crate::cli_util::CommandHelper;
-use crate::command_error::user_error;
 use crate::command_error::CommandError;
+use crate::command_error::user_error;
 use crate::config::ConfigEnv;
 use crate::ui::Ui;
 
@@ -67,33 +67,42 @@ impl ConfigLevelArgs {
         }
     }
 
-    fn config_path<'a>(&self, config_env: &'a ConfigEnv) -> Result<&'a Path, CommandError> {
+    fn config_paths<'a>(&self, config_env: &'a ConfigEnv) -> Result<Vec<&'a Path>, CommandError> {
         if self.user {
-            config_env
-                .user_config_path()
-                .ok_or_else(|| user_error("No user config path found"))
+            let paths = config_env.user_config_paths().collect_vec();
+            if paths.is_empty() {
+                return Err(user_error("No user config path found"));
+            }
+            Ok(paths)
         } else if self.repo {
             config_env
                 .repo_config_path()
+                .map(|p| vec![p])
                 .ok_or_else(|| user_error("No repo config path found"))
         } else {
             panic!("No config_level provided")
         }
     }
 
-    fn edit_config_file(&self, command: &CommandHelper) -> Result<ConfigFile, CommandError> {
+    fn edit_config_file(
+        &self,
+        ui: &Ui,
+        command: &CommandHelper,
+    ) -> Result<ConfigFile, CommandError> {
         let config_env = command.config_env();
         let config = command.raw_config();
         let pick_one = |mut files: Vec<ConfigFile>, not_found_error: &str| {
             if files.len() > 1 {
-                // TODO: prompt or pick the last?
-                return Err(user_error(format!(
-                    "Cannot determine config file to edit:\n{}",
-                    files
-                        .iter()
-                        .map(|file| format!("  {}", file.path().display()))
-                        .join("\n")
-                )));
+                let mut choices = vec![];
+                let mut formatter = ui.stderr_formatter();
+                for (i, file) in files.iter().enumerate() {
+                    writeln!(formatter, "{}: {}", i + 1, file.path().display())?;
+                    choices.push((i + 1).to_string());
+                }
+                drop(formatter);
+                let index =
+                    ui.prompt_choice("Choose a config file (default 1)", &choices, Some(0))?;
+                return Ok(files[index].clone());
             }
             files.pop().ok_or_else(|| user_error(not_found_error))
         };
