@@ -19,6 +19,11 @@ use itertools::Itertools as _;
 use jj_lib::config::ConfigGetResultExt as _;
 use jj_lib::git;
 use jj_lib::git::GitFetch;
+use jj_lib::git::IgnoredRefspec;
+use jj_lib::git::IgnoredRefspecs;
+use jj_lib::git::expand_default_fetch_refspecs;
+use jj_lib::git::expand_fetch_refspecs;
+use jj_lib::git::get_git_backend;
 use jj_lib::ref_name::RemoteName;
 use jj_lib::repo::Repo as _;
 use jj_lib::str_util::StringPattern;
@@ -45,15 +50,21 @@ pub struct GitFetchArgs {
     ///
     /// By default, the specified name matches exactly. Use `glob:` prefix to
     /// expand `*` as a glob, e.g. `--branch 'glob:push-*'`. Other wildcard
-    /// characters such as `?` are *not* supported.
+    /// characters such as `?` are *not* supported. Can be repeated to specify
+    /// multiple branches.
     #[arg(
         long, short,
         alias = "bookmark",
-        default_value = "glob:*",
         value_parser = StringPattern::parse,
         add = ArgValueCandidates::new(complete::bookmarks),
     )]
     branch: Vec<StringPattern>,
+    /// Fetch only tracked bookmarks
+    ///
+    /// This fetches only bookmarks that are already tracked from the specified
+    /// remote(s).
+    #[arg(long, conflicts_with = "branch")]
+    tracked: bool,
     /// The remote to fetch from (only named remotes are supported, can be
     /// repeated)
     ///
@@ -119,7 +130,45 @@ pub fn cmd_git_fetch(
         .collect_vec();
 
     let mut tx = workspace_command.start_transaction();
-    do_git_fetch(ui, &mut tx, &remotes, &args.branch)?;
+
+    let mut expansions = Vec::with_capacity(remotes.len());
+    if args.tracked {
+        for remote in &remotes {
+            let tracked_branches = tx
+                .repo()
+                .view()
+                .local_remote_bookmarks(remote)
+                .filter(|(_, targets)| targets.remote_ref.is_tracked())
+                .map(|(name, _)| StringPattern::exact(name))
+                .collect_vec();
+            expansions.push((remote, expand_fetch_refspecs(remote, tracked_branches)?));
+        }
+    } else if args.branch.is_empty() {
+        let git_repo = get_git_backend(tx.repo_mut().store())?.git_repo();
+        for remote in &remotes {
+            let (ignored, expanded) = expand_default_fetch_refspecs(remote, &git_repo)?;
+            warn_ignored_refspecs(ui, remote, ignored)?;
+            expansions.push((remote, expanded));
+        }
+    } else {
+        for remote in &remotes {
+            let expanded = expand_fetch_refspecs(remote, args.branch.clone())?;
+            expansions.push((remote, expanded));
+        }
+    };
+
+    let git_settings = tx.settings().git_settings()?;
+    let mut git_fetch = GitFetch::new(tx.repo_mut(), &git_settings)?;
+
+    for (remote, expanded) in expansions {
+        with_remote_git_callbacks(ui, |callbacks| {
+            git_fetch.fetch(remote, expanded, callbacks, None, None)
+        })?;
+    }
+
+    let import_stats = git_fetch.import_refs()?;
+    print_git_import_stats(ui, tx.repo(), &import_stats, true)?;
+    warn_if_branches_not_found(ui, &tx, &args.branch, &remotes)?;
     tx.finish(
         ui,
         format!(
@@ -164,25 +213,6 @@ fn parse_remote_pattern(remote: &str) -> Result<StringPattern, CommandError> {
     StringPattern::parse(remote).map_err(config_error)
 }
 
-fn do_git_fetch(
-    ui: &mut Ui,
-    tx: &mut WorkspaceCommandTransaction,
-    remotes: &[&RemoteName],
-    branch_names: &[StringPattern],
-) -> Result<(), CommandError> {
-    let git_settings = tx.settings().git_settings()?;
-    let mut git_fetch = GitFetch::new(tx.repo_mut(), &git_settings)?;
-
-    for remote_name in remotes {
-        with_remote_git_callbacks(ui, |callbacks| {
-            git_fetch.fetch(remote_name, branch_names, callbacks, None)
-        })?;
-    }
-    let import_stats = git_fetch.import_refs()?;
-    print_git_import_stats(ui, tx.repo(), &import_stats, true)?;
-    warn_if_branches_not_found(ui, tx, branch_names, remotes)
-}
-
 fn warn_if_branches_not_found(
     ui: &mut Ui,
     tx: &WorkspaceCommandTransaction,
@@ -220,3 +250,19 @@ fn warn_if_branches_not_found(
 
     Ok(())
 }
+
+fn warn_ignored_refspecs(
+    ui: &Ui,
+    remote_name: &RemoteName,
+    IgnoredRefspecs(ignored_refspecs): IgnoredRefspecs,
+) -> Result<(), CommandError> {
+    let remote_name = remote_name.as_symbol();
+    for IgnoredRefspec { refspec, reason } in ignored_refspecs {
+        writeln!(
+            ui.warning_default(),
+            "Ignored refspec `{refspec}` from `{remote_name}`: {reason}",
+        )?;
+    }
+
+    Ok(())
+}
