@@ -16,7 +16,6 @@ use std::io;
 use std::io::Write as _;
 use std::path::Path;
 use std::path::PathBuf;
-use std::str;
 use std::sync::Arc;
 
 use indoc::writedoc;
@@ -40,6 +39,7 @@ use crate::command_error::internal_error;
 use crate::command_error::user_error_with_hint;
 use crate::command_error::user_error_with_message;
 use crate::commands::git::maybe_add_gitignore;
+use crate::formatter::FormatterExt as _;
 use crate::git_util::is_colocated_git_workspace;
 use crate::git_util::print_git_export_stats;
 use crate::git_util::print_git_import_stats;
@@ -78,7 +78,7 @@ pub struct GitInitArgs {
     /// If the specified `--git-repo` path happens to be the same as
     /// the `jj` repo path (both .jj and .git directories are in the
     /// same working directory), then both `jj` and `git` commands
-    /// will work on the same repo. This is called a co-located repo.
+    /// will work on the same repo. This is called a colocated repo.
     ///
     /// This option is mutually exclusive with `--colocate`.
     #[arg(long, conflicts_with = "colocate", value_hint = clap::ValueHint::DirPath)]
@@ -259,16 +259,15 @@ pub fn maybe_set_repository_level_trunk_alias(
             // we should stop here and not try other remotes because it doesn't
             // really make sense if "origin" were to be set as the default if we
             // know "upstream" exists.
-            if let Some(reference_name) = reference.target().try_name() {
-                if let Some((GitRefKind::Bookmark, symbol)) =
+            if let Some(reference_name) = reference.target().try_name()
+                && let Some((GitRefKind::Bookmark, symbol)) =
                     str::from_utf8(reference_name.as_bstr())
                         .ok()
                         .and_then(|name| parse_git_ref(name.as_ref()))
-                {
-                    // TODO: Can we assume the symbolic target points to the same remote?
-                    let symbol = symbol.name.to_remote_symbol(remote.as_ref());
-                    write_repository_level_trunk_alias(ui, workspace_command.repo_path(), symbol)?;
-                }
+            {
+                // TODO: Can we assume the symbolic target points to the same remote?
+                let symbol = symbol.name.to_remote_symbol(remote.as_ref());
+                write_repository_level_trunk_alias(ui, workspace_command.repo_path(), symbol)?;
             }
             return Ok(());
         }
