@@ -44,6 +44,7 @@ use crate::diff_util::DiffFormatArgs;
 use crate::diff_util::DiffRenderer;
 use crate::diff_util::diff_formats_for_log;
 use crate::formatter::Formatter;
+use crate::formatter::FormatterExt as _;
 use crate::graphlog::GraphStyle;
 use crate::graphlog::get_graphlog;
 use crate::templater::TemplateRenderer;
@@ -317,12 +318,13 @@ pub fn show_op_diff(
         }
     }
 
-    let changed_tags =
-        diff_named_ref_targets(from_repo.view().tags(), to_repo.view().tags()).collect_vec();
-    if !changed_tags.is_empty() {
+    let changed_local_tags =
+        diff_named_ref_targets(from_repo.view().local_tags(), to_repo.view().local_tags())
+            .collect_vec();
+    if !changed_local_tags.is_empty() {
         writeln!(formatter)?;
         with_content_format.write(formatter, |formatter| writeln!(formatter, "Changed tags:"))?;
-        for (name, (from_target, to_target)) in changed_tags {
+        for (name, (from_target, to_target)) in changed_local_tags {
             with_content_format.write(formatter, |formatter| {
                 writeln!(formatter, "{name}:", name = name.as_symbol())?;
                 write_ref_target_summary(
@@ -396,15 +398,13 @@ fn write_modified_change_summary(
     modified_change: &ModifiedChange,
 ) -> Result<(), std::io::Error> {
     for commit in modified_change.added_commits() {
-        formatter.with_label("diff", |formatter| write!(formatter.labeled("added"), "+"))?;
+        write!(formatter.labeled("diff").labeled("added"), "+")?;
         write!(formatter, " ")?;
         commit_summary_template.format(commit, formatter)?;
         writeln!(formatter)?;
     }
     for commit in modified_change.removed_commits() {
-        formatter.with_label("diff", |formatter| {
-            write!(formatter.labeled("removed"), "-")
-        })?;
+        write!(formatter.labeled("diff").labeled("removed"), "-")?;
         write!(formatter, " ")?;
         commit_summary_template.format(commit, formatter)?;
         writeln!(formatter)?;
@@ -425,13 +425,11 @@ fn write_ref_target_summary(
                         added: bool,
                         prefix: Option<&str>|
      -> Result<(), CommandError> {
-        formatter.with_label("diff", |formatter| {
-            write!(
-                formatter.labeled(if added { "added" } else { "removed" }),
-                "{}",
-                if added { "+" } else { "-" }
-            )
-        })?;
+        if added {
+            write!(formatter.labeled("diff").labeled("added"), "+")?;
+        } else {
+            write!(formatter.labeled("diff").labeled("removed"), "-")?;
+        }
         write!(formatter, " ")?;
         if let Some(prefix) = prefix {
             write!(formatter, "{prefix} ")?;
