@@ -24,6 +24,7 @@ use crate::cli_util::print_conflicted_paths;
 use crate::command_error::CommandError;
 use crate::command_error::cli_error;
 use crate::complete;
+use crate::formatter::FormatterExt as _;
 use crate::ui::Ui;
 
 /// Resolve conflicted files with an external merge tool
@@ -50,7 +51,7 @@ pub(crate) struct ResolveArgs {
         long, short,
         default_value = "@",
         value_name = "REVSET",
-        add = ArgValueCompleter::new(complete::revset_expression_mutable),
+        add = ArgValueCompleter::new(complete::revset_expression_mutable_conflicts),
     )]
     revision: RevisionArg,
     /// Instead of resolving conflicts, list all the conflicts
@@ -132,18 +133,17 @@ pub(crate) fn cmd_resolve(
     // Print conflicts that are still present after resolution if the workspace
     // working copy is not at the commit. Otherwise, the conflicting paths will
     // be printed by the `tx.finish()` instead.
-    if workspace_command.get_wc_commit_id() != Some(new_commit.id()) {
-        if let Some(mut formatter) = ui.status_formatter() {
-            if new_commit.has_conflict()? {
-                let new_tree = new_commit.tree()?;
-                let new_conflicts = new_tree.conflicts().collect_vec();
-                writeln!(
-                    formatter.labeled("warning").with_heading("Warning: "),
-                    "After this operation, some files at this revision still have conflicts:"
-                )?;
-                print_conflicted_paths(new_conflicts, formatter.as_mut(), &workspace_command)?;
-            }
-        }
+    if workspace_command.get_wc_commit_id() != Some(new_commit.id())
+        && let Some(mut formatter) = ui.status_formatter()
+        && new_commit.has_conflict()?
+    {
+        let new_tree = new_commit.tree()?;
+        let new_conflicts = new_tree.conflicts().collect_vec();
+        writeln!(
+            formatter.labeled("warning").with_heading("Warning: "),
+            "After this operation, some files at this revision still have conflicts:"
+        )?;
+        print_conflicted_paths(new_conflicts, formatter.as_mut(), &workspace_command)?;
     }
 
     if let Some(err) = partial_resolution_error {
