@@ -162,7 +162,8 @@ pub(crate) fn cmd_restore(
             to_commit = workspace_command.format_commit_summary(&to_commit),
         }
     };
-    let new_tree_id = diff_selector.select(&to_tree, &from_tree, &matcher, format_instructions)?;
+    let new_tree_id =
+        diff_selector.select([&to_tree, &from_tree], &matcher, format_instructions)?;
     if &new_tree_id == to_commit.tree_id() {
         writeln!(ui.status(), "Nothing changed.")?;
     } else {
@@ -181,13 +182,13 @@ pub(crate) fn cmd_restore(
         } else {
             (tx.repo_mut().rebase_descendants()?, "")
         };
-        if let Some(mut formatter) = ui.status_formatter() {
-            if num_rebased > 0 {
-                writeln!(
-                    formatter,
-                    "Rebased {num_rebased} descendant commits{extra_msg}"
-                )?;
-            }
+        if let Some(mut formatter) = ui.status_formatter()
+            && num_rebased > 0
+        {
+            writeln!(
+                formatter,
+                "Rebased {num_rebased} descendant commits{extra_msg}"
+            )?;
         }
         tx.finish(ui, format!("restore into commit {}", to_commit.id().hex()))?;
     }
