@@ -13,7 +13,9 @@
 // limitations under the License.
 
 use clap_complete::ArgValueCandidates;
+use clap_complete::ArgValueCompleter;
 use jj_lib::matchers::EverythingMatcher;
+use pollster::FutureExt as _;
 use tracing::instrument;
 
 use crate::cli_util::CommandHelper;
@@ -25,12 +27,14 @@ use crate::ui::Ui;
 
 /// Show commit description and changes in a revision
 #[derive(clap::Args, Clone, Debug)]
+#[command(mut_arg("ignore_all_space", |a| a.short('w')))]
+#[command(mut_arg("ignore_space_change", |a| a.short('b')))]
 pub(crate) struct ShowArgs {
     /// Show changes in this revision, compared to its parent(s)
     #[arg(
         default_value = "@",
         value_name = "REVSET",
-        add = ArgValueCandidates::new(complete::all_revisions)
+        add = ArgValueCompleter::new(complete::revset_expression_all),
     )]
     revision: RevisionArg,
     /// Ignored (but lets you pass `-r` for consistency with other commands)
@@ -50,6 +54,9 @@ pub(crate) struct ShowArgs {
     template: Option<String>,
     #[command(flatten)]
     format: DiffFormatArgs,
+    /// Do not show the patch
+    #[arg(long, conflicts_with = "DiffFormatArgs")]
+    no_patch: bool,
 }
 
 #[instrument(skip_all)]
@@ -61,15 +68,21 @@ pub(crate) fn cmd_show(
     let workspace_command = command.workspace_helper(ui)?;
     let commit = workspace_command.resolve_single_rev(ui, &args.revision)?;
     let template_string = match &args.template {
-        Some(value) => value.to_string(),
+        Some(value) => value.clone(),
         None => workspace_command.settings().get_string("templates.show")?,
     };
-    let template = workspace_command.parse_commit_template(ui, &template_string)?;
+    let template = workspace_command
+        .parse_commit_template(ui, &template_string)?
+        .labeled(["show", "commit"]);
     let diff_renderer = workspace_command.diff_renderer_for(&args.format)?;
     ui.request_pager();
     let mut formatter = ui.stdout_formatter();
     let formatter = formatter.as_mut();
     template.format(&commit, formatter)?;
-    diff_renderer.show_patch(ui, formatter, &commit, &EverythingMatcher, ui.term_width())?;
+    if !args.no_patch {
+        diff_renderer
+            .show_patch(ui, formatter, &commit, &EverythingMatcher, ui.term_width())
+            .block_on()?;
+    }
     Ok(())
 }
