@@ -465,8 +465,7 @@ The changes that are not selected will replace the original commit.
     };
     let parent_tree = target_commit.parent_tree(tx.repo())?;
     let selected_tree_id = diff_selector.select(
-        &parent_tree,
-        &target_commit.tree()?,
+        [&parent_tree, &target_commit.tree()?],
         matcher,
         format_instructions,
     )?;
