@@ -15,6 +15,7 @@
 use tracing::instrument;
 
 use crate::cli_util::CommandHelper;
+use crate::cli_util::print_snapshot_stats;
 use crate::command_error::CommandError;
 use crate::ui::Ui;
 
@@ -33,7 +34,8 @@ pub fn cmd_workspace_update_stale(
     command: &CommandHelper,
     _args: &WorkspaceUpdateStaleArgs,
 ) -> Result<(), CommandError> {
-    command.recover_stale_working_copy(ui)?;
+    let (workspace_command, stats) = command.recover_stale_working_copy(ui)?;
+    print_snapshot_stats(ui, &stats, workspace_command.env().path_converter())?;
 
     Ok(())
 }
