@@ -68,7 +68,7 @@ pub struct ExternalMergeTool {
     /// paths to the corresponding files.
     pub merge_args: Vec<String>,
     /// By default, if a merge tool exits with a non-zero exit code, then the
-    /// merge will be cancelled. Some merge tools allow leaving some conflicts
+    /// merge will be canceled. Some merge tools allow leaving some conflicts
     /// unresolved, in which case they will be left as conflict markers in the
     /// output file. In that case, the merge tool may exit with a non-zero exit
     /// code to indicate that not all conflicts were resolved. Adding an exit
@@ -192,10 +192,6 @@ fn run_mergetool_external_single_file(
         file,
     } = merge_tool_file;
 
-    let conflict_marker_style = editor
-        .conflict_marker_style
-        .unwrap_or(default_conflict_marker_style);
-
     let uses_marker_length = find_all_variables(&editor.merge_args).contains(&"marker_length");
 
     // If the merge tool doesn't get conflict markers pre-populated in the output
@@ -209,8 +205,11 @@ fn run_mergetool_external_single_file(
     };
     let initial_output_content = if editor.merge_tool_edits_conflict_markers {
         let options = ConflictMaterializeOptions {
-            marker_style: conflict_marker_style,
+            marker_style: editor
+                .conflict_marker_style
+                .unwrap_or(default_conflict_marker_style),
             marker_len: Some(conflict_marker_len),
+            merge: store.merge_options().clone(),
         };
         materialize_merge_result_to_bytes(&file.contents, &options)
     } else {
@@ -253,6 +252,7 @@ fn run_mergetool_external_single_file(
         })
         .try_collect()?;
     variables.insert("marker_length", conflict_marker_len.to_string());
+    variables.insert("path", repo_path.as_internal_file_string().to_string());
 
     let mut cmd = Command::new(&editor.program);
     cmd.args(interpolate_variables(&editor.merge_args, &variables));
@@ -294,7 +294,6 @@ fn run_mergetool_external_single_file(
             store,
             repo_path,
             output_file_contents.as_slice(),
-            conflict_marker_style,
             conflict_marker_len,
         )
         .block_on()?
@@ -378,8 +377,7 @@ pub fn run_mergetool_external(
 
 pub fn edit_diff_external(
     editor: &ExternalMergeTool,
-    left_tree: &MergedTree,
-    right_tree: &MergedTree,
+    trees: [&MergedTree; 2],
     matcher: &dyn Matcher,
     instructions: Option<&str>,
     base_ignores: Arc<GitIgnoreFile>,
@@ -395,11 +393,8 @@ pub fn edit_diff_external(
     } else {
         DiffType::TwoWay
     };
-    let store = left_tree.store();
     let diffedit_wc = DiffEditWorkingCopies::check_out(
-        store,
-        left_tree,
-        right_tree,
+        trees,
         matcher,
         diff_type,
         instructions,
@@ -429,33 +424,21 @@ pub fn edit_diff_external(
 pub fn generate_diff(
     ui: &Ui,
     writer: &mut dyn Write,
-    left_tree: &MergedTree,
-    right_tree: &MergedTree,
+    trees: [&MergedTree; 2],
     matcher: &dyn Matcher,
     tool: &ExternalMergeTool,
     default_conflict_marker_style: ConflictMarkerStyle,
+    width: usize,
 ) -> Result<(), DiffGenerateError> {
     let conflict_marker_style = tool
         .conflict_marker_style
         .unwrap_or(default_conflict_marker_style);
-    let store = left_tree.store();
-    let diff_wc = check_out_trees(
-        store,
-        left_tree,
-        right_tree,
-        matcher,
-        DiffType::TwoWay,
-        conflict_marker_style,
-    )?;
+    let diff_wc = check_out_trees(trees, matcher, DiffType::TwoWay, conflict_marker_style)?;
     diff_wc.set_left_readonly()?;
     diff_wc.set_right_readonly()?;
-    invoke_external_diff(
-        ui,
-        writer,
-        tool,
-        diff_wc.temp_dir(),
-        &diff_wc.to_command_variables(true),
-    )
+    let mut patterns = diff_wc.to_command_variables(true);
+    patterns.insert("width", width.to_string());
+    invoke_external_diff(ui, writer, tool, diff_wc.temp_dir(), &patterns)
 }
 
 /// Invokes the specified `tool` directing its output into `writer`.
@@ -464,24 +447,26 @@ pub fn invoke_external_diff(
     writer: &mut dyn Write,
     tool: &ExternalMergeTool,
     diff_dir: &Path,
-    patterns: &HashMap<&str, &str>,
+    patterns: &HashMap<&str, String>,
 ) -> Result<(), DiffGenerateError> {
     // TODO: Somehow propagate --color to the external command?
     let mut cmd = Command::new(&tool.program);
     let mut patterns = patterns.clone();
-    let absolute_left_path = Path::new(diff_dir).join(patterns["left"]);
-    let absolute_right_path = Path::new(diff_dir).join(patterns["right"]);
     if !tool.diff_do_chdir {
+        let absolute_left_path = Path::new(diff_dir).join(&patterns["left"]);
+        let absolute_right_path = Path::new(diff_dir).join(&patterns["right"]);
         patterns.insert(
             "left",
             absolute_left_path
-                .to_str()
+                .into_os_string()
+                .into_string()
                 .expect("temp_dir should be valid utf-8"),
         );
         patterns.insert(
             "right",
             absolute_right_path
-                .to_str()
+                .into_os_string()
+                .into_string()
                 .expect("temp_dir should be valid utf-8"),
         );
     } else {
