@@ -32,7 +32,8 @@ use jj_lib::op_store::TimestampRange;
 
 use crate::formatter::FormatRecorder;
 use crate::formatter::Formatter;
-use crate::formatter::LabeledWriter;
+use crate::formatter::FormatterExt as _;
+use crate::formatter::LabeledScope;
 use crate::formatter::PlainTextFormatter;
 use crate::text_util;
 use crate::time_util;
@@ -52,11 +53,6 @@ pub trait ListTemplate: Template {
     fn join<'a>(self: Box<Self>, separator: Box<dyn Template + 'a>) -> Box<dyn Template + 'a>
     where
         Self: 'a;
-
-    /// Upcasts to the template type.
-    fn into_template<'a>(self: Box<Self>) -> Box<dyn Template + 'a>
-    where
-        Self: 'a;
 }
 
 impl<T: Template + ?Sized> Template for &T {
@@ -594,13 +590,6 @@ where
             self.format_item,
         ))
     }
-
-    fn into_template<'a>(self: Box<Self>) -> Box<dyn Template + 'a>
-    where
-        Self: 'a,
-    {
-        self
-    }
 }
 
 /// Template which selects an output based on a boolean condition.
@@ -799,19 +788,16 @@ impl<'a> TemplateFormatter<'a> {
         self.formatter.raw()
     }
 
-    pub fn labeled<S: AsRef<str>>(
-        &mut self,
-        label: S,
-    ) -> LabeledWriter<&mut (dyn Formatter + 'a), S> {
+    pub fn labeled(&mut self, label: &str) -> LabeledScope<&mut (dyn Formatter + 'a)> {
         self.formatter.labeled(label)
     }
 
-    pub fn push_label(&mut self, label: &str) -> io::Result<()> {
-        self.formatter.push_label(label)
+    pub fn push_label(&mut self, label: &str) {
+        self.formatter.push_label(label);
     }
 
-    pub fn pop_label(&mut self) -> io::Result<()> {
-        self.formatter.pop_label()
+    pub fn pop_label(&mut self) {
+        self.formatter.pop_label();
     }
 
     pub fn write_fmt(&mut self, args: fmt::Arguments<'_>) -> io::Result<()> {
@@ -880,11 +866,11 @@ fn format_labeled<T: Template + ?Sized>(
     labels: &[String],
 ) -> io::Result<()> {
     for label in labels {
-        formatter.push_label(label)?;
+        formatter.push_label(label);
     }
     content.format(formatter)?;
     for _label in labels {
-        formatter.pop_label()?;
+        formatter.pop_label();
     }
     Ok(())
 }
@@ -897,16 +883,15 @@ fn format_property_error_inline(
     err: TemplatePropertyError,
 ) -> io::Result<()> {
     let TemplatePropertyError(err) = &err;
-    formatter.with_label("error", |formatter| {
-        write!(formatter, "<")?;
-        write!(formatter.labeled("heading"), "Error: ")?;
-        write!(formatter, "{err}")?;
-        for err in iter::successors(err.source(), |err| err.source()) {
-            write!(formatter, ": {err}")?;
-        }
-        write!(formatter, ">")?;
-        Ok(())
-    })
+    let mut formatter = formatter.labeled("error");
+    write!(formatter, "<")?;
+    write!(formatter.labeled("heading"), "Error: ")?;
+    write!(formatter, "{err}")?;
+    for err in iter::successors(err.source(), |err| err.source()) {
+        write!(formatter, ": {err}")?;
+    }
+    write!(formatter, ">")?;
+    Ok(())
 }
 
 fn propagate_property_error(
