@@ -14,6 +14,7 @@ mod test_acls;
 mod test_advance_bookmarks;
 mod test_alias;
 mod test_backout_command;
+mod test_bisect_command;
 mod test_bookmark_command;
 mod test_builtin_aliases;
 mod test_commit_command;
@@ -37,6 +38,7 @@ mod test_file_show_command;
 mod test_file_track_untrack_commands;
 mod test_fix_command;
 mod test_generate_md_cli_help;
+mod test_gerrit_upload;
 mod test_git_clone;
 mod test_git_colocated;
 mod test_git_fetch;
