@@ -50,6 +50,7 @@ fn test_bookmark_multiple_names() {
     let output = work_dir.run_jj(["bookmark", "create", "-r@", "foo", "bar"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
+    Warning: Target revision is empty.
     Created 2 bookmarks pointing to qpvuntsm e8849ae1 bar foo | (empty) (no description set)
     [EOF]
     ");
@@ -60,9 +61,10 @@ fn test_bookmark_multiple_names() {
     ");
 
     work_dir.run_jj(["new"]).success();
-    let output = work_dir.run_jj(["bookmark", "set", "foo", "bar", "--to=@"]);
+    let output = work_dir.run_jj(["bookmark", "set", "foo", "bar"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
+    Warning: Target revision is empty.
     Moved 2 bookmarks to zsuskuln 0e555a27 bar foo | (empty) (no description set)
     [EOF]
     ");
@@ -90,6 +92,7 @@ fn test_bookmark_multiple_names() {
     let output = work_dir.run_jj(["bookmark", "create", "-r@-", "foo", "bar"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
+    Warning: Target revision is empty.
     Created 2 bookmarks pointing to qpvuntsm e8849ae1 bar foo | (empty) (no description set)
     [EOF]
     ");
@@ -98,6 +101,7 @@ fn test_bookmark_multiple_names() {
     let output = work_dir.run_jj(["bookmark", "set", "-r@", "bar", "baz"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
+    Warning: Target revision is empty.
     Created 1 bookmarks pointing to zsuskuln 0e555a27 bar baz | (empty) (no description set)
     Moved 1 bookmarks to zsuskuln 0e555a27 bar baz | (empty) (no description set)
     [EOF]
@@ -107,6 +111,7 @@ fn test_bookmark_multiple_names() {
     let output = work_dir.run_jj(["bookmark", "set", "-r@", "foo", "bar", "baz"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
+    Warning: Target revision is empty.
     Moved 1 bookmarks to zsuskuln 0e555a27 bar baz foo | (empty) (no description set)
     [EOF]
     ");
@@ -121,6 +126,7 @@ fn test_bookmark_at_root() {
     let output = work_dir.run_jj(["bookmark", "create", "fred", "-r=root()"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
+    Warning: Target revision is empty.
     Created 1 bookmarks pointing to zzzzzzzz 00000000 fred | (empty) (no description set)
     [EOF]
     ");
@@ -140,7 +146,7 @@ fn test_bookmark_bad_name() {
     test_env.run_jj_in(".", ["git", "init", "repo"]).success();
     let work_dir = test_env.work_dir("repo");
 
-    let output = work_dir.run_jj(["bookmark", "create", "-r@", ""]);
+    let output = work_dir.run_jj(["bookmark", "create", ""]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     error: invalid value '' for '<NAMES>...': Failed to parse bookmark name: Syntax error
@@ -227,9 +233,10 @@ fn test_bookmark_bad_name() {
     ");
 
     // quoted name works
-    let output = work_dir.run_jj(["bookmark", "create", "-r@", "'foo@bar'"]);
+    let output = work_dir.run_jj(["bookmark", "create", "'foo@bar'"]);
     insta::assert_snapshot!(output, @r#"
     ------- stderr -------
+    Warning: Target revision is empty.
     Created 1 bookmarks pointing to qpvuntsm e8849ae1 "foo@bar" | (empty) (no description set)
     [EOF]
     "#);
@@ -248,7 +255,7 @@ fn test_bookmark_move() {
         .run_jj(["git", "remote", "add", "origin", "../git-repo"])
         .success();
 
-    let output = work_dir.run_jj(["bookmark", "move", "foo", "--to=@"]);
+    let output = work_dir.run_jj(["bookmark", "move", "foo"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Error: No such bookmark: foo
@@ -256,15 +263,16 @@ fn test_bookmark_move() {
     [exit status: 1]
     ");
 
-    let output = work_dir.run_jj(["bookmark", "set", "foo", "--to=@"]);
+    let output = work_dir.run_jj(["bookmark", "set", "foo"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
+    Warning: Target revision is empty.
     Created 1 bookmarks pointing to qpvuntsm e8849ae1 foo | (empty) (no description set)
     [EOF]
     ");
 
     work_dir.run_jj(["new"]).success();
-    let output = work_dir.run_jj(["bookmark", "create", "-r@", "foo"]);
+    let output = work_dir.run_jj(["bookmark", "create", "foo"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Error: Bookmark already exists: foo
@@ -276,6 +284,7 @@ fn test_bookmark_move() {
     let output = work_dir.run_jj(["bookmark", "set", "foo", "--revision", "@"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
+    Warning: Target revision is empty.
     Moved 1 bookmarks to mzvwutvl 8afc18ff foo | (empty) (no description set)
     [EOF]
     ");
@@ -292,13 +301,15 @@ fn test_bookmark_move() {
     let output = work_dir.run_jj(["bookmark", "set", "-r@-", "--allow-backwards", "foo"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
+    Warning: Target revision is empty.
     Moved 1 bookmarks to qpvuntsm e8849ae1 foo | (empty) (no description set)
     [EOF]
     ");
 
-    let output = work_dir.run_jj(["bookmark", "move", "foo", "--to=@"]);
+    let output = work_dir.run_jj(["bookmark", "move", "foo"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
+    Warning: Target revision is empty.
     Moved 1 bookmarks to mzvwutvl 8afc18ff foo | (empty) (no description set)
     [EOF]
     ");
@@ -315,6 +326,7 @@ fn test_bookmark_move() {
     let output = work_dir.run_jj(["bookmark", "move", "--to=@-", "--allow-backwards", "foo"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
+    Warning: Target revision is empty.
     Moved 1 bookmarks to qpvuntsm e8849ae1 foo | (empty) (no description set)
     [EOF]
     ");
@@ -332,7 +344,7 @@ fn test_bookmark_move() {
     ");
 
     // Deleted tracking bookmark name should still be allocated
-    let output = work_dir.run_jj(["bookmark", "create", "-r@", "foo"]);
+    let output = work_dir.run_jj(["bookmark", "create", "foo"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Error: Tracked remote bookmarks exist for deleted bookmark: foo
@@ -342,9 +354,10 @@ fn test_bookmark_move() {
     ");
 
     // Restoring local target shouldn't invalidate tracking state
-    let output = work_dir.run_jj(["bookmark", "set", "foo", "--to=@"]);
+    let output = work_dir.run_jj(["bookmark", "set", "foo"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
+    Warning: Target revision is empty.
     Moved 1 bookmarks to mzvwutvl 91b59745 foo* | (empty) (no description set)
     [EOF]
     ");
@@ -359,9 +372,10 @@ fn test_bookmark_move() {
         .run_jj(["bookmark", "untrack", "foo@origin"])
         .success();
     work_dir.run_jj(["bookmark", "delete", "foo"]).success();
-    let output = work_dir.run_jj(["bookmark", "create", "-r@", "foo"]);
+    let output = work_dir.run_jj(["bookmark", "create", "foo"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
+    Warning: Target revision is empty.
     Created 1 bookmarks pointing to mzvwutvl 91b59745 foo | (empty) (no description set)
     [EOF]
     ");
@@ -379,17 +393,13 @@ fn test_bookmark_move_matching() {
     let work_dir = test_env.work_dir("repo");
 
     work_dir
-        .run_jj(["bookmark", "create", "-r@", "a1", "a2"])
+        .run_jj(["bookmark", "create", "a1", "a2"])
         .success();
     work_dir.run_jj(["new", "-mhead1"]).success();
     work_dir.run_jj(["new", "root()"]).success();
-    work_dir
-        .run_jj(["bookmark", "create", "-r@", "b1"])
-        .success();
+    work_dir.run_jj(["bookmark", "create", "b1"]).success();
     work_dir.run_jj(["new"]).success();
-    work_dir
-        .run_jj(["bookmark", "create", "-r@", "c1"])
-        .success();
+    work_dir.run_jj(["bookmark", "create", "c1"]).success();
     work_dir.run_jj(["new", "-mhead2"]).success();
     insta::assert_snapshot!(get_log_output(&work_dir), @r"
     @   0dd9a4b12283
@@ -404,13 +414,13 @@ fn test_bookmark_move_matching() {
     let setup_opid = work_dir.current_operation_id();
 
     // The default could be considered "--from=all() glob:*", but is disabled
-    let output = work_dir.run_jj(["bookmark", "move", "--to=@"]);
+    let output = work_dir.run_jj(["bookmark", "move"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     error: the following required arguments were not provided:
       <NAMES|--from <REVSETS>>
 
-    Usage: jj bookmark move --to <REVSET> <NAMES|--from <REVSETS>>
+    Usage: jj bookmark move <NAMES|--from <REVSETS>>
 
     For more information, try '--help'.
     [EOF]
@@ -418,7 +428,7 @@ fn test_bookmark_move_matching() {
     ");
 
     // No bookmarks pointing to the source revisions
-    let output = work_dir.run_jj(["bookmark", "move", "--from=none()", "--to=@"]);
+    let output = work_dir.run_jj(["bookmark", "move", "--from=none()"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     No bookmarks to update.
@@ -426,7 +436,7 @@ fn test_bookmark_move_matching() {
     ");
 
     // No matching bookmarks within the source revisions
-    let output = work_dir.run_jj(["bookmark", "move", "--from=::@", "glob:a?", "--to=@"]);
+    let output = work_dir.run_jj(["bookmark", "move", "--from=::@", "glob:a?"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Error: No matching bookmarks for patterns: a?
@@ -443,7 +453,7 @@ fn test_bookmark_move_matching() {
     ");
 
     // Move from multiple revisions
-    let output = work_dir.run_jj(["bookmark", "move", "--from=::@", "--to=@"]);
+    let output = work_dir.run_jj(["bookmark", "move", "--from=::@"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Moved 2 bookmarks to vruxwmqv 0dd9a4b1 b1 c1 | (empty) head2
@@ -463,7 +473,7 @@ fn test_bookmark_move_matching() {
     work_dir.run_jj(["op", "restore", &setup_opid]).success();
 
     // Try to move multiple bookmarks, but one of them isn't fast-forward
-    let output = work_dir.run_jj(["bookmark", "move", "glob:?1", "--to=@"]);
+    let output = work_dir.run_jj(["bookmark", "move", "glob:?1"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Error: Refusing to move bookmark backwards or sideways: a1
@@ -597,17 +607,13 @@ fn test_bookmark_rename() {
     ");
 
     work_dir.run_jj(["describe", "-m=commit-0"]).success();
-    work_dir
-        .run_jj(["bookmark", "create", "-r@", "blocal"])
-        .success();
+    work_dir.run_jj(["bookmark", "create", "blocal"]).success();
     let output = work_dir.run_jj(["bookmark", "rename", "blocal", "blocal1"]);
     insta::assert_snapshot!(output, @"");
 
     work_dir.run_jj(["new"]).success();
     work_dir.run_jj(["describe", "-m=commit-1"]).success();
-    work_dir
-        .run_jj(["bookmark", "create", "-r@", "bexist"])
-        .success();
+    work_dir.run_jj(["bookmark", "create", "bexist"]).success();
     let output = work_dir.run_jj(["bookmark", "rename", "blocal1", "bexist"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
@@ -618,9 +624,7 @@ fn test_bookmark_rename() {
 
     work_dir.run_jj(["new"]).success();
     work_dir.run_jj(["describe", "-m=commit-2"]).success();
-    work_dir
-        .run_jj(["bookmark", "create", "-r@", "bremote"])
-        .success();
+    work_dir.run_jj(["bookmark", "create", "bremote"]).success();
     work_dir
         .run_jj(["git", "push", "--allow-new", "-b=bremote"])
         .success();
@@ -649,9 +653,7 @@ fn test_bookmark_rename_colocated() {
     let work_dir = test_env.work_dir("repo");
 
     work_dir.run_jj(["describe", "-m=commit-0"]).success();
-    work_dir
-        .run_jj(["bookmark", "create", "-r@", "blocal"])
-        .success();
+    work_dir.run_jj(["bookmark", "create", "blocal"]).success();
 
     // Make sure that git tracking bookmarks don't cause a warning
     let output = work_dir.run_jj(["bookmark", "rename", "blocal", "blocal1"]);
@@ -664,18 +666,10 @@ fn test_bookmark_forget_glob() {
     test_env.run_jj_in(".", ["git", "init", "repo"]).success();
     let work_dir = test_env.work_dir("repo");
 
-    work_dir
-        .run_jj(["bookmark", "create", "-r@", "foo-1"])
-        .success();
-    work_dir
-        .run_jj(["bookmark", "create", "-r@", "bar-2"])
-        .success();
-    work_dir
-        .run_jj(["bookmark", "create", "-r@", "foo-3"])
-        .success();
-    work_dir
-        .run_jj(["bookmark", "create", "-r@", "foo-4"])
-        .success();
+    work_dir.run_jj(["bookmark", "create", "foo-1"]).success();
+    work_dir.run_jj(["bookmark", "create", "bar-2"]).success();
+    work_dir.run_jj(["bookmark", "create", "foo-3"]).success();
+    work_dir.run_jj(["bookmark", "create", "foo-4"]).success();
     let setup_opid = work_dir.current_operation_id();
 
     insta::assert_snapshot!(get_log_output(&work_dir), @r"
@@ -758,18 +752,10 @@ fn test_bookmark_delete_glob() {
         .success();
 
     work_dir.run_jj(["describe", "-m=commit"]).success();
-    work_dir
-        .run_jj(["bookmark", "create", "-r@", "foo-1"])
-        .success();
-    work_dir
-        .run_jj(["bookmark", "create", "-r@", "bar-2"])
-        .success();
-    work_dir
-        .run_jj(["bookmark", "create", "-r@", "foo-3"])
-        .success();
-    work_dir
-        .run_jj(["bookmark", "create", "-r@", "foo-4"])
-        .success();
+    work_dir.run_jj(["bookmark", "create", "foo-1"]).success();
+    work_dir.run_jj(["bookmark", "create", "bar-2"]).success();
+    work_dir.run_jj(["bookmark", "create", "foo-3"]).success();
+    work_dir.run_jj(["bookmark", "create", "foo-4"]).success();
     // Push to create remote-tracking bookmarks
     work_dir.run_jj(["git", "push", "--all"]).success();
     let setup_opid = work_dir.current_operation_id();
@@ -866,9 +852,7 @@ fn test_bookmark_delete_export() {
     let work_dir = test_env.work_dir("repo");
 
     work_dir.run_jj(["new"]).success();
-    work_dir
-        .run_jj(["bookmark", "create", "-r@", "foo"])
-        .success();
+    work_dir.run_jj(["bookmark", "create", "foo"]).success();
     work_dir.run_jj(["git", "export"]).success();
 
     work_dir.run_jj(["bookmark", "delete", "foo"]).success();
@@ -893,9 +877,7 @@ fn test_bookmark_forget_export() {
     let work_dir = test_env.work_dir("repo");
 
     work_dir.run_jj(["new"]).success();
-    work_dir
-        .run_jj(["bookmark", "create", "-r@", "foo"])
-        .success();
+    work_dir.run_jj(["bookmark", "create", "foo"]).success();
     insta::assert_snapshot!(get_bookmark_output(&work_dir), @r"
     foo: rlvkpnrz 43444d88 (empty) (no description set)
     [EOF]
@@ -1331,9 +1313,7 @@ fn test_bookmark_track_conflict() {
     work_dir
         .run_jj(["git", "remote", "add", "origin", "../git-repo"])
         .success();
-    work_dir
-        .run_jj(["bookmark", "create", "-r@", "main"])
-        .success();
+    work_dir.run_jj(["bookmark", "create", "main"]).success();
     work_dir.run_jj(["describe", "-m", "a"]).success();
     work_dir
         .run_jj(["git", "push", "--allow-new", "-b", "main"])
@@ -1388,9 +1368,7 @@ fn test_bookmark_track_untrack_patterns() {
     ");
 
     // Track local bookmark
-    work_dir
-        .run_jj(["bookmark", "create", "-r@", "main"])
-        .success();
+    work_dir.run_jj(["bookmark", "create", "main"]).success();
     insta::assert_snapshot!(work_dir.run_jj(["bookmark", "track", "main"]), @r"
     ------- stderr -------
     error: invalid value 'main' for '<BOOKMARK@REMOTE>...': remote bookmark must be specified in bookmark@remote form
@@ -1524,7 +1502,7 @@ fn test_bookmark_list() {
             .run_jj(["new", "root()", "-m", bookmark])
             .success();
         remote_dir
-            .run_jj(["bookmark", "create", "-r@", bookmark])
+            .run_jj(["bookmark", "create", bookmark])
             .success();
     }
     remote_dir.run_jj(["new"]).success();
@@ -1544,7 +1522,7 @@ fn test_bookmark_list() {
         .run_jj(["new", "root()", "-m", "local-only"])
         .success();
     local_dir
-        .run_jj(["bookmark", "create", "-r@", "local-only"])
+        .run_jj(["bookmark", "create", "local-only"])
         .success();
 
     // Mutate refs in local repository
@@ -1558,13 +1536,7 @@ fn test_bookmark_list() {
         .run_jj(["bookmark", "untrack", "remote-untrack@origin"])
         .success();
     local_dir
-        .run_jj([
-            "bookmark",
-            "set",
-            "--allow-backwards",
-            "--to=@",
-            "remote-unsync",
-        ])
+        .run_jj(["bookmark", "set", "--allow-backwards", "remote-unsync"])
         .success();
 
     // Synchronized tracking remotes and non-tracking remotes aren't listed by
@@ -1746,7 +1718,7 @@ fn test_bookmark_list_filtered() {
             .run_jj(["new", "root()", "-m", bookmark])
             .success();
         remote_dir
-            .run_jj(["bookmark", "create", "-r@", bookmark])
+            .run_jj(["bookmark", "create", bookmark])
             .success();
     }
     remote_dir.run_jj(["new"]).success();
@@ -1766,7 +1738,7 @@ fn test_bookmark_list_filtered() {
         .run_jj(["new", "root()", "-m", "local-keep"])
         .success();
     local_dir
-        .run_jj(["bookmark", "create", "-r@", "local-keep"])
+        .run_jj(["bookmark", "create", "local-keep"])
         .success();
 
     // Mutate refs in local repository
@@ -2025,18 +1997,12 @@ fn test_bookmark_list_much_remote_divergence() {
         local_dir.run_jj(["new", "-m", "local-only"]).success();
     }
     local_dir
-        .run_jj(["bookmark", "create", "-r@", "local-only"])
+        .run_jj(["bookmark", "create", "local-only"])
         .success();
 
     // Mutate refs in local repository
     local_dir
-        .run_jj([
-            "bookmark",
-            "set",
-            "--allow-backwards",
-            "--to=@",
-            "remote-unsync",
-        ])
+        .run_jj(["bookmark", "set", "--allow-backwards", "remote-unsync"])
         .success();
 
     let output = local_dir.run_jj(["bookmark", "list"]);
@@ -2066,7 +2032,7 @@ fn test_bookmark_list_tracked() {
             .run_jj(["new", "root()", "-m", bookmark])
             .success();
         remote_dir
-            .run_jj(["bookmark", "create", "-r@", bookmark])
+            .run_jj(["bookmark", "create", bookmark])
             .success();
     }
     remote_dir.run_jj(["new"]).success();
@@ -2098,7 +2064,7 @@ fn test_bookmark_list_tracked() {
         .run_jj(["new", "root()", "-m", "upstream-sync"])
         .success();
     upstream_dir
-        .run_jj(["bookmark", "create", "-r@", "upstream-sync"])
+        .run_jj(["bookmark", "create", "upstream-sync"])
         .success();
     upstream_dir.run_jj(["new"]).success();
     upstream_dir.run_jj(["git", "export"]).success();
@@ -2125,7 +2091,7 @@ fn test_bookmark_list_tracked() {
         .run_jj(["new", "root()", "-m", "local-only"])
         .success();
     local_dir
-        .run_jj(["bookmark", "create", "-r@", "local-only"])
+        .run_jj(["bookmark", "create", "local-only"])
         .success();
 
     // Mutate refs in local repository
@@ -2150,13 +2116,7 @@ fn test_bookmark_list_tracked() {
         ])
         .success();
     local_dir
-        .run_jj([
-            "bookmark",
-            "set",
-            "--to=@",
-            "--allow-backwards",
-            "remote-unsync",
-        ])
+        .run_jj(["bookmark", "set", "--allow-backwards", "remote-unsync"])
         .success();
 
     let output = local_dir.run_jj(["bookmark", "list", "--all-remotes"]);
@@ -2245,9 +2205,7 @@ fn test_bookmark_list_conflicted() {
     // Track existing bookmark. Local bookmark should result in conflict.
     work_dir.run_jj(["new", "root()", "-m", "a"]).success();
     work_dir.run_jj(["new", "root()", "-m", "b"]).success();
-    work_dir
-        .run_jj(["bookmark", "create", "-r@", "bar"])
-        .success();
+    work_dir.run_jj(["bookmark", "create", "bar"]).success();
     work_dir
         .run_jj(["bookmark", "create", "foo", "-r", "description(a)"])
         .success();
@@ -2309,9 +2267,7 @@ fn test_bookmark_list_sort_multiple_keys() {
                 "fix",
             ])
             .success();
-        work_dir
-            .run_jj(["bookmark", "create", "-r@", bookmark])
-            .success();
+        work_dir.run_jj(["bookmark", "create", bookmark]).success();
     }
 
     let template =
@@ -2339,9 +2295,7 @@ fn test_bookmark_list_sort_using_config() {
                 "fix",
             ])
             .success();
-        work_dir
-            .run_jj(["bookmark", "create", "-r@", bookmark])
-            .success();
+        work_dir.run_jj(["bookmark", "create", bookmark]).success();
     }
 
     let template = r#"name ++ ": " ++ if(normal_target, normal_target.author().email()) ++ "\n""#;
@@ -2374,9 +2328,7 @@ fn test_bookmark_list_sort_overriding_config() {
                 "fix",
             ])
             .success();
-        work_dir
-            .run_jj(["bookmark", "create", "-r@", bookmark])
-            .success();
+        work_dir.run_jj(["bookmark", "create", bookmark]).success();
     }
 
     let template = r#"name ++ ": " ++ if(normal_target, normal_target.author().email()) ++ "\n""#;
@@ -2395,66 +2347,6 @@ fn test_bookmark_list_sort_overriding_config() {
     ");
 }
 
-#[test]
-fn test_bookmark_create_with_default_target_revision() {
-    let test_env = TestEnvironment::default();
-    test_env.run_jj_in(".", ["git", "init", "repo"]).success();
-    let work_dir = test_env.work_dir("repo");
-
-    let output = work_dir.run_jj(["bookmark", "create", "foo"]);
-    insta::assert_snapshot!(output, @r"
-    ------- stderr -------
-    Warning: Target revision was not specified, defaulting to the working copy (-r@). In the near future it will be required to explicitly specify target revision.
-    Created 1 bookmarks pointing to qpvuntsm e8849ae1 foo | (empty) (no description set)
-    [EOF]
-    ");
-}
-
-#[test]
-fn test_bookmark_set_with_default_target_revision() {
-    let test_env = TestEnvironment::default();
-    test_env.run_jj_in(".", ["git", "init", "repo"]).success();
-    let work_dir = test_env.work_dir("repo");
-
-    let output = work_dir.run_jj(["bookmark", "set", "foo"]);
-    insta::assert_snapshot!(output, @r"
-    ------- stderr -------
-    Warning: Target revision was not specified, defaulting to the working copy (--revision=@). In the near future it will be required to explicitly specify target revision.
-    Created 1 bookmarks pointing to qpvuntsm e8849ae1 foo | (empty) (no description set)
-    [EOF]
-    ");
-}
-
-#[test]
-fn test_bookmark_move_with_default_target_revision() {
-    let test_env = TestEnvironment::default();
-    test_env.run_jj_in(".", ["git", "init", "repo"]).success();
-    let work_dir = test_env.work_dir("repo");
-
-    // Set up remote
-    let git_repo_path = test_env.env_root().join("git-repo");
-    git::init_bare(git_repo_path);
-    work_dir
-        .run_jj(["git", "remote", "add", "origin", "../git-repo"])
-        .success();
-
-    let output = work_dir.run_jj(["bookmark", "create", "foo", "-r@"]);
-    insta::assert_snapshot!(output, @r"
-    ------- stderr -------
-    Created 1 bookmarks pointing to qpvuntsm e8849ae1 foo | (empty) (no description set)
-    [EOF]
-    ");
-
-    work_dir.run_jj(["new"]).success();
-    let output = work_dir.run_jj(["bookmark", "move", "foo"]);
-    insta::assert_snapshot!(output, @r"
-    ------- stderr -------
-    Warning: Target revision was not specified, defaulting to the working copy (--to=@). In the near future it will be required to explicitly specify it.
-    Moved 1 bookmarks to zsuskuln 0e555a27 foo | (empty) (no description set)
-    [EOF]
-    ");
-}
-
 #[must_use]
 fn get_log_output(work_dir: &TestWorkDir) -> CommandOutput {
     let template = r#"bookmarks ++ " " ++ commit_id.short()"#;
