@@ -643,6 +643,9 @@ fn test_config_set_for_user_directory() {
 
     [template-aliases]
     'format_time_range(time_range)' = 'time_range.start() ++ " - " ++ time_range.end()'
+
+    [git]
+    colocate = false
     "#);
 
     // Add one more config file to the directory
@@ -666,6 +669,9 @@ fn test_config_set_for_user_directory() {
 
     [template-aliases]
     'format_time_range(time_range)' = 'time_range.start() ++ " - " ++ time_range.end()'
+
+    [git]
+    colocate = false
     "#);
 
     insta::assert_snapshot!(
@@ -1246,26 +1252,16 @@ fn test_config_get() {
     ");
 
     let output = test_env.run_jj_in(".", ["config", "get", "table.list"]);
-    insta::assert_snapshot!(output, @r"
-    ------- stderr -------
-    Config error: Invalid type or value for table.list
-    Caused by: Expected a value convertible to a string, but is an array
-    Hint: Check the config file: $TEST_ENV/config/config0002.toml
-    For help, see https://jj-vcs.github.io/jj/latest/config/ or use `jj help -k config`.
+    insta::assert_snapshot!(output, @r#"
+    ["list", "value"]
     [EOF]
-    [exit status: 1]
-    ");
+    "#);
 
     let output = test_env.run_jj_in(".", ["config", "get", "table"]);
-    insta::assert_snapshot!(output, @r"
-    ------- stderr -------
-    Config error: Invalid type or value for table
-    Caused by: Expected a value convertible to a string, but is a table
-    Hint: Check the config file: $TEST_ENV/config/config0003.toml
-    For help, see https://jj-vcs.github.io/jj/latest/config/ or use `jj help -k config`.
+    insta::assert_snapshot!(output, @r#"
+    { string = "some value 1", int = 123, list = ["list", "value"], overridden = "bar" }
     [EOF]
-    [exit status: 1]
-    ");
+    "#);
 
     let output = test_env.run_jj_in(".", ["config", "get", "table.overridden"]);
     insta::assert_snapshot!(output, @r"
@@ -1276,7 +1272,16 @@ fn test_config_get() {
 
 #[test]
 fn test_config_get_yields_values_consistent_with_schema_defaults() {
-    let test_env = TestEnvironment::default();
+    let mut test_env = TestEnvironment::default();
+
+    // The default test environment may already contain configuration that's
+    // different from the true default, e.g. `git.colocate = false`. So we
+    // explicitly set the config to an empty one in order to test the true
+    // default config values.
+    let config_dir = test_env.env_root().join("empty-config");
+    std::fs::create_dir(&config_dir).unwrap();
+    test_env.set_config_path(&config_dir);
+
     let get_true_default = move |key: &str| {
         let output = test_env.run_jj_in(".", ["config", "get", key]).success();
         let output_doc = toml_edit::Document::parse(format!("test={}", output.stdout.normalized()))
@@ -1694,7 +1699,7 @@ fn test_config_author_change_warning() {
     ------- stderr -------
     Warning: This setting will only impact future commits.
     The author of the working copy will stay "Test User <test.user@example.com>".
-    To change the working copy author, use "jj describe --reset-author --no-edit"
+    To change the working copy author, use "jj metaedit --update-author"
     [EOF]
     "#);
 
@@ -1702,14 +1707,14 @@ fn test_config_author_change_warning() {
     // for this test, the state (user.email) is needed
     work_dir
         .run_jj_with(|cmd| {
-            cmd.args(["describe", "--reset-author", "--no-edit"])
+            cmd.args(["metaedit", "--update-author"])
                 .env_remove("JJ_EMAIL")
         })
         .success();
 
     let output = work_dir.run_jj(["log"]);
     insta::assert_snapshot!(output, @r"
-    @  qpvuntsm Foo 2001-02-03 08:05:09 f64cf908
+    @  qpvuntsm Foo 2001-02-03 08:05:09 c2090b51
     │  (empty) (no description set)
     ◆  zzzzzzzz root() 00000000
     [EOF]
