@@ -21,7 +21,7 @@ fn test_interdiff_basic() {
     let work_dir = test_env.work_dir("repo");
 
     work_dir.write_file("file1", "foo\n");
-    work_dir.run_jj(["new"]).success();
+    work_dir.run_jj(["new", "-madd file2 left"]).success();
     work_dir.write_file("file2", "foo\n");
     work_dir
         .run_jj(["bookmark", "create", "-r@", "left"])
@@ -29,15 +29,18 @@ fn test_interdiff_basic() {
 
     work_dir.run_jj(["new", "root()"]).success();
     work_dir.write_file("file3", "foo\n");
-    work_dir.run_jj(["new"]).success();
+    work_dir.run_jj(["new", "-madd file2 right"]).success();
     work_dir.write_file("file2", "foo\nbar\n");
     work_dir
         .run_jj(["bookmark", "create", "-r@", "right"])
         .success();
+    let setup_opid = work_dir.current_operation_id();
 
     // implicit --to
     let output = work_dir.run_jj(["interdiff", "--from", "left"]);
     insta::assert_snapshot!(output, @r"
+    Modified commit description:
+       1    1: add file2 leftright
     Modified regular file file2:
        1    1: foo
             2: bar
@@ -48,12 +51,14 @@ fn test_interdiff_basic() {
     work_dir.run_jj(["new", "@-"]).success();
     let output = work_dir.run_jj(["interdiff", "--from", "left", "--to", "right"]);
     insta::assert_snapshot!(output, @r"
+    Modified commit description:
+       1    1: add file2 leftright
     Modified regular file file2:
        1    1: foo
             2: bar
     [EOF]
     ");
-    work_dir.run_jj(["undo"]).success();
+    work_dir.run_jj(["op", "restore", &setup_opid]).success();
 
     // formats specifiers
     let output = work_dir.run_jj(["interdiff", "--from", "left", "--to", "right", "-s"]);
@@ -64,6 +69,12 @@ fn test_interdiff_basic() {
 
     let output = work_dir.run_jj(["interdiff", "--from", "left", "--to", "right", "--git"]);
     insta::assert_snapshot!(output, @r"
+    diff --git a/JJ-COMMIT-DESCRIPTION b/JJ-COMMIT-DESCRIPTION
+    --- JJ-COMMIT-DESCRIPTION
+    +++ JJ-COMMIT-DESCRIPTION
+    @@ -1,1 +1,1 @@
+    -add file2 left
+    +add file2 right
     diff --git a/file2 b/file2
     index 257cc5642c..3bd1f0e297 100644
     --- a/file2
@@ -163,4 +174,24 @@ fn test_interdiff_conflicting() {
     +def
     [EOF]
     ");
+
+    let output = work_dir.run_jj([
+        "interdiff",
+        "--config=diff.color-words.conflict=pair",
+        "--color=always",
+        "--from=left",
+        "--to=right",
+    ]);
+    insta::assert_snapshot!(output, @r"
+    [38;5;3mResolved conflict in file:[39m
+    [38;5;6m<<<<<<< Resolved conflict[39m
+    [38;5;6m+++++++ left side #1 to right side #1[39m
+    [38;5;1m   1[39m [38;5;2m   1[39m: [4m[38;5;1mabc[38;5;2mdef[24m[39m
+    [38;5;6m------- left base #1 to right side #1[39m
+    [38;5;2m   1[39m [38;5;1m   1[39m: [4m[38;5;2mfoo[38;5;1mdef[24m[39m
+    [38;5;6m+++++++ left side #2 to right side #1[39m
+    [38;5;1m   1[39m [38;5;2m   1[39m: [4m[38;5;1mbar[38;5;2mdef[24m[39m
+    [38;5;6m>>>>>>> Conflict ends[39m
+    [EOF]
+    ");
 }
