@@ -12,7 +12,7 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
-#![allow(missing_docs)]
+#![expect(missing_docs)]
 
 use std::any::Any;
 use std::fmt::Debug;
@@ -51,7 +51,6 @@ id_type!(
 id_type!(pub TreeId { hex() });
 id_type!(pub FileId { hex() });
 id_type!(pub SymlinkId { hex() });
-id_type!(pub ConflictId { hex() });
 id_type!(pub CopyId { hex() });
 
 impl ChangeId {
@@ -208,21 +207,6 @@ pub struct Commit {
     pub secure_sig: Option<SecureSig>,
 }
 
-#[derive(ContentHash, Debug, PartialEq, Eq, Clone)]
-pub struct ConflictTerm {
-    pub value: TreeValue,
-}
-
-#[derive(ContentHash, Debug, PartialEq, Eq, Clone)]
-pub struct Conflict {
-    // A conflict is represented by a list of positive and negative states that need to be applied.
-    // In a simple 3-way merge of B and C with merge base A, the conflict will be { add: [B, C],
-    // remove: [A] }. Also note that a conflict of the form { add: [A], remove: [] } is the
-    // same as non-conflict A.
-    pub removes: Vec<ConflictTerm>,
-    pub adds: Vec<ConflictTerm>,
-}
-
 /// An individual copy event, from file A -> B.
 #[derive(Debug, PartialEq, Eq, Clone)]
 pub struct CopyRecord {
@@ -351,7 +335,6 @@ pub enum TreeValue {
     Symlink(SymlinkId),
     Tree(TreeId),
     GitSubmodule(CommitId),
-    Conflict(ConflictId),
 }
 
 impl TreeValue {
@@ -361,7 +344,6 @@ impl TreeValue {
             Self::Symlink(id) => id.hex(),
             Self::Tree(id) => id.hex(),
             Self::GitSubmodule(id) => id.hex(),
-            Self::Conflict(id) => id.hex(),
         }
     }
 }
@@ -463,9 +445,7 @@ pub fn make_root_commit(root_change_id: ChangeId, empty_tree_id: TreeId) -> Comm
 
 /// Defines the interface for commit backends.
 #[async_trait]
-pub trait Backend: Send + Sync + Debug {
-    fn as_any(&self) -> &dyn Any;
-
+pub trait Backend: Any + Send + Sync + Debug {
     /// A unique name that identifies this backend. Written to
     /// `.jj/repo/store/type` when the repo is created.
     fn name(&self) -> &str;
@@ -534,12 +514,6 @@ pub trait Backend: Send + Sync + Debug {
 
     async fn write_tree(&self, path: &RepoPath, contents: &Tree) -> BackendResult<TreeId>;
 
-    // Not async because it would force `MergedTree::value()` to be async. We don't
-    // need this to be async anyway because it's only used by legacy repos.
-    fn read_conflict(&self, path: &RepoPath, id: &ConflictId) -> BackendResult<Conflict>;
-
-    fn write_conflict(&self, path: &RepoPath, contents: &Conflict) -> BackendResult<ConflictId>;
-
     async fn read_commit(&self, id: &CommitId) -> BackendResult<Commit>;
 
     /// Writes a commit and returns its ID and the commit itself. The commit
@@ -587,3 +561,10 @@ pub trait Backend: Send + Sync + Debug {
     /// risk of deleting new commits created concurrently by another process.
     fn gc(&self, index: &dyn Index, keep_newer: SystemTime) -> BackendResult<()>;
 }
+
+impl dyn Backend {
+    /// Returns reference of the implementation type.
+    pub fn downcast_ref<T: Backend>(&self) -> Option<&T> {
+        (self as &dyn Any).downcast_ref()
+    }
+}
