@@ -32,8 +32,8 @@ use itertools::Itertools as _;
 use serde::Deserialize;
 use serde::de::IntoDeserializer as _;
 use thiserror::Error;
+use toml_edit::Document;
 use toml_edit::DocumentMut;
-use toml_edit::ImDocument;
 
 pub use crate::config_resolver::ConfigMigrateError;
 pub use crate::config_resolver::ConfigMigrateLayerError;
@@ -336,7 +336,7 @@ impl ConfigLayer {
 
     /// Parses TOML document `text` into new layer.
     pub fn parse(source: ConfigSource, text: &str) -> Result<Self, ConfigLoadError> {
-        let data = ImDocument::parse(text).map_err(|error| ConfigLoadError::Parse {
+        let data = Document::parse(text).map_err(|error| ConfigLoadError::Parse {
             error: Box::new(error),
             source_path: None,
         })?;
@@ -348,7 +348,7 @@ impl ConfigLayer {
         let text = fs::read_to_string(&path)
             .context(&path)
             .map_err(ConfigLoadError::Read)?;
-        let data = ImDocument::parse(text).map_err(|error| ConfigLoadError::Parse {
+        let data = Document::parse(text).map_err(|error| ConfigLoadError::Parse {
             error: Box::new(error),
             source_path: Some(path.clone()),
         })?;
@@ -523,33 +523,21 @@ fn look_up_item<'a>(
 /// Inserts tables recursively. Returns `Err(keys)` if middle node exists at the
 /// prefix name `keys` and wasn't a table.
 fn ensure_table<'a, 'b>(
-    root_table: &'a mut ConfigTable,
+    root_table: &'a mut ConfigTableLike<'a>,
     keys: &'b [toml_edit::Key],
 ) -> Result<&'a mut ConfigTableLike<'a>, &'b [toml_edit::Key]> {
-    let (table, _is_inline) = keys.iter().enumerate().try_fold(
-        (root_table as &mut ConfigTableLike, false),
-        |(table, is_inline), (i, key)| {
-            // As of toml_edit 0.22.27, InlineTable::entry_format() doesn't
-            // convert an Item to Value.
-            let sub_item = table
-                .entry_format(key)
-                .or_insert_with(|| new_implicit_table(is_inline));
-            let sub_is_inline = sub_item.is_inline_table();
-            let sub_table = sub_item.as_table_like_mut().ok_or(&keys[..=i])?;
-            Ok((sub_table, sub_is_inline))
-        },
-    )?;
-    Ok(table)
+    keys.iter()
+        .enumerate()
+        .try_fold(root_table, |table, (i, key)| {
+            let sub_item = table.entry_format(key).or_insert_with(new_implicit_table);
+            sub_item.as_table_like_mut().ok_or(&keys[..=i])
+        })
 }
 
-fn new_implicit_table(is_inline: bool) -> ConfigItem {
+fn new_implicit_table() -> ConfigItem {
     let mut table = ConfigTable::new();
     table.set_implicit(true);
-    if is_inline {
-        ConfigItem::Value(ConfigValue::InlineTable(table.into_inline_table()))
-    } else {
-        ConfigItem::Table(table)
-    }
+    ConfigItem::Table(table)
 }
 
 /// Wrapper for file-based [`ConfigLayer`], providing convenient methods for
@@ -568,15 +556,13 @@ impl ConfigFile {
     ) -> Result<Self, ConfigLoadError> {
         let layer = match ConfigLayer::load_from_file(source, path.into()) {
             Ok(layer) => Arc::new(layer),
-            Err(ConfigLoadError::Read(PathError { path, error }))
-                if error.kind() == io::ErrorKind::NotFound =>
-            {
+            Err(ConfigLoadError::Read(PathError {
+                path,
+                source: error,
+            })) if error.kind() == io::ErrorKind::NotFound => {
                 let mut data = DocumentMut::new();
-                data.insert(
-                    "$schema",
-                    toml_edit::Item::Value(
-                        "https://jj-vcs.github.io/jj/latest/config-schema.json".into(),
-                    ),
+                data.decor_mut().set_prefix(
+                    "#:schema https://jj-vcs.github.io/jj/latest/config-schema.json\n\n",
                 );
                 let layer = ConfigLayer {
                     source,
@@ -1021,7 +1007,7 @@ mod tests {
 
         // Should create nested inline tables
         layer.set_value("a.c.d", "a.c.d").unwrap();
-        insta::assert_snapshot!(layer.data, @r#"a = { b = "a.b", c = { d = "a.c.d" } }"#);
+        insta::assert_snapshot!(layer.data, @r#"a = { b = "a.b", c.d = "a.c.d" }"#);
     }
 
     #[test]
