@@ -12,7 +12,7 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
-#![allow(missing_docs)]
+#![expect(missing_docs)]
 
 use std::io;
 use std::io::Write;
@@ -40,15 +40,17 @@ use crate::backend::TreeId;
 use crate::backend::TreeValue;
 use crate::copies::CopiesTreeDiffEntry;
 use crate::copies::CopiesTreeDiffEntryPath;
-use crate::diff::Diff;
+use crate::diff::ContentDiff;
 use crate::diff::DiffHunk;
 use crate::diff::DiffHunkKind;
 use crate::files;
 use crate::files::MergeResult;
 use crate::merge::Merge;
 use crate::merge::MergedTreeValue;
+use crate::merge::SameChange;
 use crate::repo_path::RepoPath;
 use crate::store::Store;
+use crate::tree_merge::MergeOptions;
 
 /// Minimum length of conflict markers.
 pub const MIN_CONFLICT_MARKER_LEN: usize = 7;
@@ -235,9 +237,6 @@ async fn materialize_tree_value_no_access_denied(
         }
         Ok(Some(TreeValue::GitSubmodule(id))) => Ok(MaterializedTreeValue::GitSubmodule(id)),
         Ok(Some(TreeValue::Tree(id))) => Ok(MaterializedTreeValue::Tree(id)),
-        Ok(Some(TreeValue::Conflict(_))) => {
-            panic!("cannot materialize legacy conflict object at path {path:?}");
-        }
         Err(conflict) => match try_materialize_file_conflict_value(store, path, &conflict).await? {
             Some(file) => Ok(MaterializedTreeValue::FileConflict(file)),
             None => Ok(MaterializedTreeValue::OtherConflict { id: conflict }),
@@ -272,7 +271,7 @@ pub async fn try_materialize_file_conflict_value(
 /// Resolves conflicts in file executable bit, returns the original state if the
 /// file is deleted and executable bit is unchanged.
 pub fn resolve_file_executable(merge: &Merge<Option<bool>>) -> Option<bool> {
-    let resolved = merge.resolve_trivial().copied()?;
+    let resolved = merge.resolve_trivial(SameChange::Accept).copied()?;
     if resolved.is_some() {
         resolved
     } else {
@@ -284,11 +283,10 @@ pub fn resolve_file_executable(merge: &Merge<Option<bool>>) -> Option<bool> {
 }
 
 /// Describes what style should be used when materializing conflicts.
-#[derive(Clone, Copy, PartialEq, Eq, Debug, Default, serde::Deserialize)]
+#[derive(Clone, Copy, PartialEq, Eq, Debug, serde::Deserialize)]
 #[serde(rename_all = "kebab-case")]
 pub enum ConflictMarkerStyle {
     /// Style which shows a snapshot and a series of diffs to apply.
-    #[default]
     Diff,
     /// Style which shows a snapshot for each base and side.
     Snapshot,
@@ -297,10 +295,11 @@ pub enum ConflictMarkerStyle {
 }
 
 /// Options for conflict materialization.
-#[derive(Clone, Debug, Default)]
+#[derive(Clone, Debug)]
 pub struct ConflictMaterializeOptions {
     pub marker_style: ConflictMarkerStyle,
     pub marker_len: Option<usize>,
+    pub merge: MergeOptions,
 }
 
 /// Characters which can be repeated to form a conflict marker line when
@@ -407,7 +406,7 @@ pub fn materialize_merge_result<T: AsRef<[u8]>>(
     output: &mut dyn Write,
     options: &ConflictMaterializeOptions,
 ) -> io::Result<()> {
-    let merge_result = files::merge_hunks(single_hunk);
+    let merge_result = files::merge_hunks(single_hunk, &options.merge);
     match &merge_result {
         MergeResult::Resolved(content) => output.write_all(content),
         MergeResult::Conflict(hunks) => {
@@ -423,7 +422,7 @@ pub fn materialize_merge_result_to_bytes<T: AsRef<[u8]>>(
     single_hunk: &Merge<T>,
     options: &ConflictMaterializeOptions,
 ) -> BString {
-    let merge_result = files::merge_hunks(single_hunk);
+    let merge_result = files::merge_hunks(single_hunk, &options.merge);
     match merge_result {
         MergeResult::Resolved(content) => content,
         MergeResult::Conflict(hunks) => {
@@ -617,12 +616,12 @@ fn materialize_jj_style_conflict(
             continue;
         }
 
-        let diff1 = Diff::by_line([&left, &right1]).hunks().collect_vec();
+        let diff1 = ContentDiff::by_line([&left, &right1]).hunks().collect_vec();
         // Check if the diff against the next positive term is better. Since we want to
         // preserve the order of the terms, we don't match against any later positive
         // terms.
         if let Some(right2) = hunk.get_add(add_index + 1) {
-            let diff2 = Diff::by_line([&left, &right2]).hunks().collect_vec();
+            let diff2 = ContentDiff::by_line([&left, &right2]).hunks().collect_vec();
             if diff_size(&diff2) < diff_size(&diff1) {
                 // If the next positive term is a better match, emit the current positive term
                 // as a snapshot and the next positive term as a diff.
@@ -698,9 +697,9 @@ pub fn materialized_diff_stream<'a>(
                 path,
                 values: Err(err),
             },
-            Ok((before, after)) => {
-                let before_future = materialize_tree_value(store, path.source(), before);
-                let after_future = materialize_tree_value(store, path.target(), after);
+            Ok(values) => {
+                let before_future = materialize_tree_value(store, path.source(), values.before);
+                let after_future = materialize_tree_value(store, path.target(), values.after);
                 let values = try_join!(before_future, after_future);
                 MaterializedTreeDiffEntry { path, values }
             }
@@ -923,50 +922,53 @@ pub async fn update_from_content(
     store: &Store,
     path: &RepoPath,
     content: &[u8],
-    conflict_marker_style: ConflictMarkerStyle,
     conflict_marker_len: usize,
 ) -> BackendResult<Merge<Option<FileId>>> {
     let simplified_file_ids = file_ids.simplify();
 
-    // First check if the new content is unchanged compared to the old content. If
-    // it is, we don't need parse the content or write any new objects to the
-    // store. This is also a way of making sure that unchanged tree/file
-    // conflicts (for example) are not converted to regular files in the working
-    // copy.
-    let mut old_content = Vec::with_capacity(content.len());
-    let merge_hunk = extract_as_single_hunk(&simplified_file_ids, store, path).await?;
-    let materialize_options = ConflictMaterializeOptions {
-        marker_style: conflict_marker_style,
-        marker_len: Some(conflict_marker_len),
-    };
-    materialize_merge_result(&merge_hunk, &mut old_content, &materialize_options).unwrap();
-    if content == old_content {
-        return Ok(file_ids.clone());
-    }
+    let old_contents = extract_as_single_hunk(&simplified_file_ids, store, path).await?;
+    let old_hunks = files::merge_hunks(&old_contents, store.merge_options());
 
     // Parse conflicts from the new content using the arity of the simplified
     // conflicts.
-    let Some(mut hunks) = parse_conflict(
+    let mut new_hunks = parse_conflict(
         content,
         simplified_file_ids.num_sides(),
         conflict_marker_len,
-    ) else {
-        // Either there are no markers or they don't have the expected arity
-        let file_id = store.write_file(path, &mut &content[..]).await?;
-        return Ok(Merge::normal(file_id));
-    };
+    );
 
     // If there is a conflict at the end of the file and a term ends with a newline,
     // check whether the original term ended with a newline. If it didn't, then
     // remove the newline since it was added automatically when materializing.
-    if let Some(last_hunk) = hunks.last_mut().filter(|hunk| !hunk.is_resolved()) {
-        for (original_content, term) in merge_hunk.iter().zip_eq(last_hunk.iter_mut()) {
+    if let Some(last_hunk) = new_hunks
+        .as_mut()
+        .and_then(|hunks| hunks.last_mut())
+        .filter(|hunk| !hunk.is_resolved())
+    {
+        for (original_content, term) in old_contents.iter().zip_eq(last_hunk.iter_mut()) {
             if term.last() == Some(&b'\n') && has_no_eol(original_content) {
                 term.pop();
             }
         }
     }
 
+    // Check if the new hunks are unchanged. This makes sure that unchanged file
+    // conflicts aren't updated to partially-resolved contents.
+    let unchanged = match (&old_hunks, &new_hunks) {
+        (MergeResult::Resolved(old), None) => old == content,
+        (MergeResult::Conflict(old), Some(new)) => old == new,
+        (MergeResult::Resolved(_), Some(_)) | (MergeResult::Conflict(_), None) => false,
+    };
+    if unchanged {
+        return Ok(file_ids.clone());
+    }
+
+    let Some(hunks) = new_hunks else {
+        // Either there are no markers or they don't have the expected arity
+        let file_id = store.write_file(path, &mut &content[..]).await?;
+        return Ok(Merge::normal(file_id));
+    };
+
     let mut contents = simplified_file_ids.map(|_| vec![]);
     for hunk in hunks {
         if let Some(slice) = hunk.as_resolved() {
