@@ -20,9 +20,9 @@ use std::fmt;
 use std::slice;
 
 use itertools::Itertools as _;
+use pest::RuleType;
 use pest::iterators::Pair;
 use pest::iterators::Pairs;
-use pest::RuleType;
 
 /// Manages diagnostic messages emitted during parsing.
 ///
@@ -37,7 +37,7 @@ pub struct Diagnostics<T> {
 impl<T> Diagnostics<T> {
     /// Creates new empty diagnostics collector.
     pub fn new() -> Self {
-        Diagnostics {
+        Self {
             diagnostics: Vec::new(),
         }
     }
@@ -97,7 +97,7 @@ pub struct ExpressionNode<'i, T> {
 impl<'i, T> ExpressionNode<'i, T> {
     /// Wraps the given expression and span.
     pub fn new(kind: T, span: pest::Span<'i>) -> Self {
-        ExpressionNode { kind, span }
+        Self { kind, span }
     }
 }
 
@@ -475,7 +475,7 @@ impl<R: RuleType> FunctionCallParser<R> {
         parse_name: impl Fn(Pair<'i, R>) -> Result<&'i str, E>,
         parse_value: impl Fn(Pair<'i, R>) -> Result<ExpressionNode<'i, T>, E>,
     ) -> Result<FunctionCallNode<'i, T>, E> {
-        let (name_pair, args_pair) = pair.into_inner().collect_tuple().unwrap();
+        let [name_pair, args_pair] = pair.into_inner().collect_array().unwrap();
         assert_eq!(name_pair.as_rule(), self.function_name_rule);
         assert_eq!(args_pair.as_rule(), self.function_arguments_rule);
         let name_span = name_pair.as_span();
@@ -496,7 +496,7 @@ impl<R: RuleType> FunctionCallParser<R> {
                 }
                 args.push(parse_value(pair)?);
             } else if pair.as_rule() == self.keyword_argument_rule {
-                let (name_pair, value_pair) = pair.into_inner().collect_tuple().unwrap();
+                let [name_pair, value_pair] = pair.into_inner().collect_array().unwrap();
                 assert_eq!(name_pair.as_rule(), self.argument_name_rule);
                 assert_eq!(value_pair.as_rule(), self.argument_value_rule);
                 let name_span = name_pair.as_span();
@@ -646,11 +646,11 @@ pub enum AliasId<'a> {
 impl fmt::Display for AliasId<'_> {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         match self {
-            AliasId::Symbol(name) => write!(f, "{name}"),
-            AliasId::Function(name, params) => {
+            Self::Symbol(name) => write!(f, "{name}"),
+            Self::Function(name, params) => {
                 write!(f, "{name}({params})", params = params.join(", "))
             }
-            AliasId::Parameter(name) => write!(f, "{name}"),
+            Self::Parameter(name) => write!(f, "{name}"),
         }
     }
 }
