@@ -12,9 +12,8 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
-#![allow(missing_docs)]
+#![expect(missing_docs)]
 
-use std::any::Any;
 use std::collections::HashSet;
 use std::ffi::OsStr;
 use std::fmt::Debug;
@@ -28,7 +27,7 @@ use std::path::PathBuf;
 use std::pin::Pin;
 use std::process::Command;
 use std::process::ExitStatus;
-use std::str;
+use std::str::Utf8Error;
 use std::sync::Arc;
 use std::sync::Mutex;
 use std::sync::MutexGuard;
@@ -57,9 +56,6 @@ use crate::backend::BackendResult;
 use crate::backend::ChangeId;
 use crate::backend::Commit;
 use crate::backend::CommitId;
-use crate::backend::Conflict;
-use crate::backend::ConflictId;
-use crate::backend::ConflictTerm;
 use crate::backend::CopyHistory;
 use crate::backend::CopyId;
 use crate::backend::CopyRecord;
@@ -80,7 +76,6 @@ use crate::file_util;
 use crate::file_util::BadPathEncoding;
 use crate::file_util::IoResultExt as _;
 use crate::file_util::PathError;
-use crate::hex_util;
 use crate::index::Index;
 use crate::lock::FileLock;
 use crate::merge::Merge;
@@ -101,7 +96,6 @@ const HASH_LENGTH: usize = 20;
 const CHANGE_ID_LENGTH: usize = 16;
 /// Ref namespace used only for preventing GC.
 const NO_GC_REF_NAMESPACE: &str = "refs/jj/keep/";
-const CONFLICT_SUFFIX: &str = ".jjconflict";
 
 pub const JJ_TREES_COMMIT_HEADER: &str = "jj:trees";
 pub const CHANGE_ID_COMMIT_HEADER: &str = "change-id";
@@ -575,12 +569,8 @@ fn commit_from_git_without_root_parent(
 
     // If the git header has a change-id field, we attempt to convert that to a
     // valid JJ Change Id
-    let change_id = commit
-        .extra_headers()
-        .find(CHANGE_ID_COMMIT_HEADER)
-        .and_then(ChangeId::try_from_reverse_hex)
-        .filter(|val| val.as_bytes().len() == CHANGE_ID_LENGTH)
-        .unwrap_or_else(|| change_id_from_git_commit_id(id));
+    let change_id = extract_change_id_from_commit(&commit)
+        .unwrap_or_else(|| synthetic_change_id_from_git_commit_id(id));
 
     // shallow commits don't have parents their parents actually fetched, so we
     // discard them here
@@ -649,7 +639,20 @@ fn commit_from_git_without_root_parent(
     })
 }
 
-fn change_id_from_git_commit_id(id: &CommitId) -> ChangeId {
+/// Extracts change id from commit headers.
+pub fn extract_change_id_from_commit(commit: &gix::objs::CommitRef) -> Option<ChangeId> {
+    commit
+        .extra_headers()
+        .find(CHANGE_ID_COMMIT_HEADER)
+        .and_then(ChangeId::try_from_reverse_hex)
+        .filter(|val| val.as_bytes().len() == CHANGE_ID_LENGTH)
+}
+
+/// Deterministically creates a change id based on the commit id
+///
+/// Used when we get a commit without a change id. The exact algorithm for the
+/// computation should not be relied upon.
+pub fn synthetic_change_id_from_git_commit_id(id: &CommitId) -> ChangeId {
     // We reverse the bits of the commit id to create the change id. We don't
     // want to use the first bytes unmodified because then it would be ambiguous
     // if a given hash prefix refers to the commit id or the change id. It would
@@ -740,21 +743,21 @@ fn deserialize_extras(commit: &mut Commit, bytes: &[u8]) {
     if !proto.change_id.is_empty() {
         commit.change_id = ChangeId::new(proto.change_id);
     }
-    if let MergedTreeId::Legacy(legacy_tree_id) = &commit.root_tree {
-        if proto.uses_tree_conflict_format {
-            if !proto.root_tree.is_empty() {
-                let merge_builder: MergeBuilder<_> = proto
-                    .root_tree
-                    .iter()
-                    .map(|id_bytes| TreeId::from_bytes(id_bytes))
-                    .collect();
-                commit.root_tree = MergedTreeId::Merge(merge_builder.build());
-            } else {
-                // uses_tree_conflict_format was set but there was no root_tree override in the
-                // proto, which means we should just promote the tree id from the
-                // git commit to be a known-conflict-free tree
-                commit.root_tree = MergedTreeId::resolved(legacy_tree_id.clone());
-            }
+    if let MergedTreeId::Legacy(legacy_tree_id) = &commit.root_tree
+        && proto.uses_tree_conflict_format
+    {
+        if !proto.root_tree.is_empty() {
+            let merge_builder: MergeBuilder<_> = proto
+                .root_tree
+                .iter()
+                .map(|id_bytes| TreeId::from_bytes(id_bytes))
+                .collect();
+            commit.root_tree = MergedTreeId::Merge(merge_builder.build());
+        } else {
+            // uses_tree_conflict_format was set but there was no root_tree override in the
+            // proto, which means we should just promote the tree id from the
+            // git commit to be a known-conflict-free tree
+            commit.root_tree = MergedTreeId::resolved(legacy_tree_id.clone());
         }
     }
     for predecessor in &proto.predecessors {
@@ -920,7 +923,7 @@ fn to_read_object_err(
     }
 }
 
-fn to_invalid_utf8_err(source: str::Utf8Error, id: &impl ObjectId) -> BackendError {
+fn to_invalid_utf8_err(source: Utf8Error, id: &impl ObjectId) -> BackendError {
     BackendError::InvalidUtf8 {
         object_type: id.object_type(),
         hash: id.hex(),
@@ -970,10 +973,6 @@ impl Debug for GitBackend {
 
 #[async_trait]
 impl Backend for GitBackend {
-    fn as_any(&self) -> &dyn Any {
-        self
-    }
-
     fn name(&self) -> &str {
         Self::name()
     }
@@ -1096,21 +1095,14 @@ impl Backend for GitBackend {
                     }
                     gix::object::tree::EntryKind::Blob => {
                         let id = FileId::from_bytes(entry.oid().as_bytes());
-                        if let Some(basename) = name.strip_suffix(CONFLICT_SUFFIX) {
-                            (
-                                basename,
-                                TreeValue::Conflict(ConflictId::from_bytes(entry.oid().as_bytes())),
-                            )
-                        } else {
-                            (
-                                name,
-                                TreeValue::File {
-                                    id,
-                                    executable: false,
-                                    copy_id: CopyId::placeholder(),
-                                },
-                            )
-                        }
+                        (
+                            name,
+                            TreeValue::File {
+                                id,
+                                executable: false,
+                                copy_id: CopyId::placeholder(),
+                            },
+                        )
                     }
                     gix::object::tree::EntryKind::BlobExecutable => {
                         let id = FileId::from_bytes(entry.oid().as_bytes());
@@ -1184,11 +1176,6 @@ impl Backend for GitBackend {
                         filename: name.into(),
                         oid: gix::ObjectId::from_bytes_or_panic(id.as_bytes()),
                     },
-                    TreeValue::Conflict(id) => gix::objs::tree::Entry {
-                        mode: gix::object::tree::EntryKind::Blob.into(),
-                        filename: (name.to_owned() + CONFLICT_SUFFIX).into(),
-                        oid: gix::ObjectId::from_bytes_or_panic(id.as_bytes()),
-                    },
                 }
             })
             .sorted_unstable()
@@ -1203,32 +1190,6 @@ impl Backend for GitBackend {
         Ok(TreeId::from_bytes(oid.as_bytes()))
     }
 
-    fn read_conflict(&self, _path: &RepoPath, id: &ConflictId) -> BackendResult<Conflict> {
-        let data = self.read_file_sync(&FileId::new(id.to_bytes()))?;
-        let json: serde_json::Value = serde_json::from_slice(&data).unwrap();
-        Ok(Conflict {
-            removes: conflict_term_list_from_json(json.get("removes").unwrap()),
-            adds: conflict_term_list_from_json(json.get("adds").unwrap()),
-        })
-    }
-
-    fn write_conflict(&self, _path: &RepoPath, conflict: &Conflict) -> BackendResult<ConflictId> {
-        let json = serde_json::json!({
-            "removes": conflict_term_list_to_json(&conflict.removes),
-            "adds": conflict_term_list_to_json(&conflict.adds),
-        });
-        let json_string = json.to_string();
-        let bytes = json_string.as_bytes();
-        let locked_repo = self.lock_git_repo();
-        let oid = locked_repo
-            .write_blob(bytes)
-            .map_err(|err| BackendError::WriteObject {
-                object_type: "conflict",
-                source: Box::new(err),
-            })?;
-        Ok(ConflictId::from_bytes(oid.as_bytes()))
-    }
-
     #[tracing::instrument(skip(self))]
     async fn read_commit(&self, id: &CommitId) -> BackendResult<Commit> {
         if *id == self.root_commit_id {
@@ -1310,11 +1271,11 @@ impl Backend for GitBackend {
             }
         }
         let mut extra_headers: Vec<(BString, BString)> = vec![];
-        if let MergedTreeId::Merge(tree_ids) = &contents.root_tree {
-            if !tree_ids.is_resolved() {
-                let value = tree_ids.iter().map(|id| id.hex()).join(" ");
-                extra_headers.push((JJ_TREES_COMMIT_HEADER.into(), value.into()));
-            }
+        if let MergedTreeId::Merge(tree_ids) = &contents.root_tree
+            && !tree_ids.is_resolved()
+        {
+            let value = tree_ids.iter().map(|id| id.hex()).join(" ");
+            extra_headers.push((JJ_TREES_COMMIT_HEADER.into(), value.into()));
         }
         if self.write_change_id_header {
             extra_headers.push((
@@ -1563,91 +1524,18 @@ recover.
     Ok(id.detach())
 }
 
-fn conflict_term_list_to_json(parts: &[ConflictTerm]) -> serde_json::Value {
-    serde_json::Value::Array(parts.iter().map(conflict_term_to_json).collect())
-}
-
-fn conflict_term_list_from_json(json: &serde_json::Value) -> Vec<ConflictTerm> {
-    json.as_array()
-        .unwrap()
-        .iter()
-        .map(conflict_term_from_json)
-        .collect()
-}
-
-fn conflict_term_to_json(part: &ConflictTerm) -> serde_json::Value {
-    serde_json::json!({
-        "value": tree_value_to_json(&part.value),
-    })
-}
-
-fn conflict_term_from_json(json: &serde_json::Value) -> ConflictTerm {
-    let json_value = json.get("value").unwrap();
-    ConflictTerm {
-        value: tree_value_from_json(json_value),
-    }
-}
-
-fn tree_value_to_json(value: &TreeValue) -> serde_json::Value {
-    match value {
-        TreeValue::File {
-            id,
-            executable,
-            copy_id: _,
-        } => serde_json::json!({
-             "file": {
-                 "id": id.hex(),
-                 "executable": executable,
-             },
-        }),
-        TreeValue::Symlink(id) => serde_json::json!({
-             "symlink_id": id.hex(),
-        }),
-        TreeValue::Tree(id) => serde_json::json!({
-             "tree_id": id.hex(),
-        }),
-        TreeValue::GitSubmodule(id) => serde_json::json!({
-             "submodule_id": id.hex(),
-        }),
-        TreeValue::Conflict(id) => serde_json::json!({
-             "conflict_id": id.hex(),
-        }),
-    }
-}
-
-fn tree_value_from_json(json: &serde_json::Value) -> TreeValue {
-    if let Some(json_file) = json.get("file") {
-        TreeValue::File {
-            id: FileId::new(bytes_vec_from_json(json_file.get("id").unwrap())),
-            executable: json_file.get("executable").unwrap().as_bool().unwrap(),
-            copy_id: CopyId::placeholder(),
-        }
-    } else if let Some(json_id) = json.get("symlink_id") {
-        TreeValue::Symlink(SymlinkId::new(bytes_vec_from_json(json_id)))
-    } else if let Some(json_id) = json.get("tree_id") {
-        TreeValue::Tree(TreeId::new(bytes_vec_from_json(json_id)))
-    } else if let Some(json_id) = json.get("submodule_id") {
-        TreeValue::GitSubmodule(CommitId::new(bytes_vec_from_json(json_id)))
-    } else if let Some(json_id) = json.get("conflict_id") {
-        TreeValue::Conflict(ConflictId::new(bytes_vec_from_json(json_id)))
-    } else {
-        panic!("unexpected json value in conflict: {json:#?}");
-    }
-}
-
-fn bytes_vec_from_json(value: &serde_json::Value) -> Vec<u8> {
-    hex_util::decode_hex(value.as_str().unwrap()).unwrap()
-}
-
 #[cfg(test)]
 mod tests {
     use assert_matches::assert_matches;
     use gix::date::parse::TimeBuf;
+    use gix::objs::CommitRef;
+    use indoc::indoc;
     use pollster::FutureExt as _;
 
     use super::*;
     use crate::config::StackedConfig;
     use crate::content_hash::blake2b_hash;
+    use crate::hex_util;
     use crate::tests::new_temp_dir;
 
     const GIT_USER: &str = "Someone";
@@ -1926,7 +1814,7 @@ mod tests {
 
         let mut commit_buf = Vec::new();
         commit.write_to(&mut commit_buf).unwrap();
-        let commit_str = std::str::from_utf8(&commit_buf).unwrap();
+        let commit_str = str::from_utf8(&commit_buf).unwrap();
 
         commit
             .extra_headers
@@ -1944,8 +1832,78 @@ mod tests {
         let sig = commit.secure_sig.expect("failed to read the signature");
 
         // converting to string for nicer assert diff
-        assert_eq!(std::str::from_utf8(&sig.sig).unwrap(), secure_sig);
-        assert_eq!(std::str::from_utf8(&sig.data).unwrap(), commit_str);
+        assert_eq!(str::from_utf8(&sig.sig).unwrap(), secure_sig);
+        assert_eq!(str::from_utf8(&sig.data).unwrap(), commit_str);
+    }
+
+    #[test]
+    fn change_id_parsing() {
+        let id = |commit_object_bytes: &[u8]| {
+            extract_change_id_from_commit(&CommitRef::from_bytes(commit_object_bytes).unwrap())
+        };
+
+        let commit_with_id = indoc! {b"
+            tree 126799bf8058d1b5c531e93079f4fe79733920dd
+            parent bd50783bdf38406dd6143475cd1a3c27938db2ee
+            author JJ Fan <jjfan@example.com> 1757112665 -0700
+            committer JJ Fan <jjfan@example.com> 1757359886 -0700
+            extra-header blah
+            change-id lkonztmnvsxytrwkxpvuutrmompwylqq
+
+            test-commit
+        "};
+        insta::assert_compact_debug_snapshot!(
+            id(commit_with_id),
+            @r#"Some(ChangeId("efbc06dc4721683f2a45568dbda31e99"))"#
+        );
+
+        let commit_without_id = indoc! {b"
+            tree 126799bf8058d1b5c531e93079f4fe79733920dd
+            parent bd50783bdf38406dd6143475cd1a3c27938db2ee
+            author JJ Fan <jjfan@example.com> 1757112665 -0700
+            committer JJ Fan <jjfan@example.com> 1757359886 -0700
+            extra-header blah
+
+            no id in header
+        "};
+        insta::assert_compact_debug_snapshot!(
+            id(commit_without_id),
+            @"None"
+        );
+
+        let commit = indoc! {b"
+            tree 126799bf8058d1b5c531e93079f4fe79733920dd
+            parent bd50783bdf38406dd6143475cd1a3c27938db2ee
+            author JJ Fan <jjfan@example.com> 1757112665 -0700
+            committer JJ Fan <jjfan@example.com> 1757359886 -0700
+            change-id lkonztmnvsxytrwkxpvuutrmompwylqq
+            extra-header blah
+            change-id abcabcabcabcabcabcabcabcabcabcab
+
+            valid change id first
+        "};
+        insta::assert_compact_debug_snapshot!(
+            id(commit),
+            @r#"Some(ChangeId("efbc06dc4721683f2a45568dbda31e99"))"#
+        );
+
+        // We only look at the first change id if multiple are present, so this should
+        // error
+        let commit = indoc! {b"
+            tree 126799bf8058d1b5c531e93079f4fe79733920dd
+            parent bd50783bdf38406dd6143475cd1a3c27938db2ee
+            author JJ Fan <jjfan@example.com> 1757112665 -0700
+            committer JJ Fan <jjfan@example.com> 1757359886 -0700
+            change-id abcabcabcabcabcabcabcabcabcabcab
+            extra-header blah
+            change-id lkonztmnvsxytrwkxpvuutrmompwylqq
+
+            valid change id first
+        "};
+        insta::assert_compact_debug_snapshot!(
+            id(commit),
+            @"None"
+        );
     }
 
     #[test]
@@ -2386,7 +2344,7 @@ mod tests {
         let obj = git_repo
             .find_object(gix::ObjectId::from_bytes_or_panic(id.as_bytes()))
             .unwrap();
-        insta::assert_snapshot!(std::str::from_utf8(&obj.data).unwrap(), @r"
+        insta::assert_snapshot!(str::from_utf8(&obj.data).unwrap(), @r"
         tree 4b825dc642cb6eb9a060e54bf8d69288fbee4904
         author Someone <someone@example.com> 0 +0000
         committer Someone <someone@example.com> 0 +0000
@@ -2404,11 +2362,11 @@ mod tests {
         let sig = commit.secure_sig.expect("failed to read the signature");
         assert_eq!(&sig, &returned_sig);
 
-        insta::assert_snapshot!(std::str::from_utf8(&sig.sig).unwrap(), @r"
+        insta::assert_snapshot!(str::from_utf8(&sig.sig).unwrap(), @r"
         test sig
         hash=03feb0caccbacce2e7b7bca67f4c82292dd487e669ed8a813120c9f82d3fd0801420a1f5d05e1393abfe4e9fc662399ec4a9a1898c5f1e547e0044a52bd4bd29
         ");
-        insta::assert_snapshot!(std::str::from_utf8(&sig.data).unwrap(), @r"
+        insta::assert_snapshot!(str::from_utf8(&sig.data).unwrap(), @r"
         tree 4b825dc642cb6eb9a060e54bf8d69288fbee4904
         author Someone <someone@example.com> 0 +0000
         committer Someone <someone@example.com> 0 +0000
