@@ -124,6 +124,10 @@ impl<'a> GitSubprocessContext<'a> {
             // In a colocated repo, the daemon will still get started the first time a `git`
             // command is run manually if the gitconfigs are set up that way.
             .args(["-c", "core.fsmonitor=false"])
+            // Avoids an error message when fetching repos with submodules if
+            // user has `submodule.recurse` configured to true in their Git
+            // config (#7565).
+            .args(["-c", "submodule.recurse=false"])
             .arg("--git-dir")
             .arg(&self.git_dir)
             // Disable translation and other locale-dependent behavior so we can
