@@ -12,15 +12,14 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
-#![allow(missing_docs)]
+#![expect(missing_docs)]
 
 use std::iter;
 use std::marker::PhantomData;
-use std::rc::Rc;
 use std::sync::Arc;
 
 use itertools::Itertools as _;
-use once_cell::unsync::OnceCell;
+use once_cell::sync::OnceCell;
 use thiserror::Error;
 
 use crate::backend::ChangeId;
@@ -47,7 +46,7 @@ pub enum IdPrefixIndexLoadError {
 }
 
 struct DisambiguationData {
-    expression: Rc<UserRevsetExpression>,
+    expression: Arc<UserRevsetExpression>,
     indexes: OnceCell<Indexes>,
 }
 
@@ -61,7 +60,7 @@ impl DisambiguationData {
     fn indexes(
         &self,
         repo: &dyn Repo,
-        extensions: &[impl AsRef<dyn SymbolResolverExtension>],
+        extensions: &[Box<dyn SymbolResolverExtension>],
     ) -> Result<&Indexes, IdPrefixIndexLoadError> {
         self.indexes.get_or_try_init(|| {
             let symbol_resolver = SymbolResolver::new(repo, extensions);
@@ -124,7 +123,7 @@ impl IdPrefixContext {
         }
     }
 
-    pub fn disambiguate_within(mut self, expression: Rc<UserRevsetExpression>) -> Self {
+    pub fn disambiguate_within(mut self, expression: Arc<UserRevsetExpression>) -> Self {
         self.disambiguation = Some(DisambiguationData {
             expression,
             indexes: OnceCell::new(),
@@ -194,13 +193,12 @@ impl IdPrefixIndex<'_> {
     }
 
     pub fn shortest_commit_prefix_len_exact(&self, repo: &dyn Repo, commit_id: &CommitId) -> usize {
-        if let Some(indexes) = self.indexes {
-            if let Some(lookup) = indexes
+        if let Some(indexes) = self.indexes
+            && let Some(lookup) = indexes
                 .commit_index
                 .lookup_exact(&*indexes.commit_change_ids, commit_id)
-            {
-                return lookup.shortest_unique_prefix_len();
-            }
+        {
+            return lookup.shortest_unique_prefix_len();
         }
         repo.index().shortest_unique_commit_id_prefix_len(commit_id)
     }
@@ -243,13 +241,12 @@ impl IdPrefixIndex<'_> {
     }
 
     fn shortest_change_prefix_len_exact(&self, repo: &dyn Repo, change_id: &ChangeId) -> usize {
-        if let Some(indexes) = self.indexes {
-            if let Some(lookup) = indexes
+        if let Some(indexes) = self.indexes
+            && let Some(lookup) = indexes
                 .change_index
                 .lookup_exact(&*indexes.commit_change_ids, change_id)
-            {
-                return lookup.shortest_unique_prefix_len();
-            }
+        {
+            return lookup.shortest_unique_prefix_len();
         }
         repo.shortest_unique_change_id_prefix_len(change_id)
     }
@@ -262,7 +259,7 @@ fn disambiguate_prefix_with_refs(view: &View, id_sym: &str, min_len: usize) -> u
             // Tags, bookmarks, and Git refs have higher priority, but Git refs
             // should include "/" char. Extension symbols have lower priority.
             let prefix = &id_sym[..n];
-            view.get_tag(prefix.as_ref()).is_absent()
+            view.get_local_tag(prefix.as_ref()).is_absent()
                 && view.get_local_bookmark(prefix.as_ref()).is_absent()
         })
         // No need to test conflicts with the full ID. We have to return some
