@@ -31,7 +31,6 @@ use itertools::Itertools as _;
 use smallvec::SmallVec;
 use smallvec::smallvec_inline;
 
-use crate::backend;
 use crate::backend::BackendResult;
 use crate::backend::CopyId;
 use crate::backend::FileId;
@@ -42,9 +41,59 @@ use crate::repo_path::RepoPath;
 use crate::store::Store;
 use crate::tree::Tree;
 
+/// A generic diff/transition from one value to another.
+///
+/// This is not a diff in the `patch(1)` sense. See `diff::ContentDiff` for
+/// that.
+#[derive(Clone, Debug, PartialEq, Eq)]
+pub struct Diff<T> {
+    /// The state before
+    pub before: T,
+    /// The state after
+    pub after: T,
+}
+
+impl<T> Diff<T> {
+    /// Create a new diff
+    pub fn new(before: T, after: T) -> Self {
+        Self { before, after }
+    }
+
+    /// Apply a function to both values
+    pub fn map<U>(self, mut f: impl FnMut(T) -> U) -> Diff<U> {
+        Diff {
+            before: f(self.before),
+            after: f(self.after),
+        }
+    }
+}
+
+impl<T: Eq> Diff<T> {
+    /// Whether the diff represents a change, i.e. if `before` and `after` are
+    /// not equal
+    pub fn is_changed(&self) -> bool {
+        self.before != self.after
+    }
+}
+
+/// Whether to resolve conflict that makes the same change at all sides.
+#[derive(Clone, Copy, Debug, Eq, PartialEq, serde::Deserialize)]
+#[serde(rename_all = "kebab-case")]
+pub enum SameChange {
+    /// Leaves same-change conflict unresolved.
+    Keep,
+    /// Resolves same-change conflict as if one side were unchanged.
+    /// (i.e. `A+(A-B)=A`)
+    ///
+    /// This matches what Git and Mercurial do (in the 3-way case at least), but
+    /// not what Darcs does. It means that repeated 3-way merging of multiple
+    /// trees may give different results depending on the order of merging.
+    Accept,
+}
+
 /// Attempt to resolve trivial conflicts between the inputs. There must be
 /// an odd number of terms.
-pub fn trivial_merge<T>(values: &[T]) -> Option<&T>
+pub fn trivial_merge<T>(values: &[T], same_change: SameChange) -> Option<&T>
 where
     T: Eq + Hash,
 {
@@ -56,7 +105,7 @@ where
     if let [add] = values {
         return Some(add);
     } else if let [add0, remove, add1] = values {
-        return if add0 == add1 {
+        return if add0 == add1 && same_change == SameChange::Accept {
             Some(add0)
         } else if add0 == remove {
             Some(add1)
@@ -76,21 +125,15 @@ where
     }
 
     // Collect non-zero value. Values with a count of 0 means that they have
-    // cancelled out.
+    // canceled out.
     counts.retain(|_, count| *count != 0);
     if counts.len() == 1 {
         // If there is a single value with a count of 1 left, then that is the result.
         let (value, count) = counts.into_iter().next().unwrap();
         assert_eq!(count, 1);
         Some(value)
-    } else if counts.len() == 2 {
+    } else if counts.len() == 2 && same_change == SameChange::Accept {
         // All sides made the same change.
-        // This matches what Git and Mercurial do (in the 3-way case at least), but not
-        // what Darcs does. It means that repeated 3-way merging of multiple trees may
-        // give different results depending on the order of merging.
-        // TODO: Consider removing this special case, making the algorithm more strict,
-        // and maybe add a more lenient version that is used when the user explicitly
-        // asks for conflict resolution.
         let [(value1, count1), (value2, count2)] = counts.into_iter().next_array().unwrap();
         assert_eq!(count1 + count2, 1);
         if count1 > 0 {
@@ -308,11 +351,11 @@ impl<T> Merge<T> {
 
     /// If this merge can be trivially resolved, returns the value it resolves
     /// to.
-    pub fn resolve_trivial(&self) -> Option<&T>
+    pub fn resolve_trivial(&self, same_change: SameChange) -> Option<&T>
     where
         T: Eq + Hash,
     {
-        trivial_merge(&self.values)
+        trivial_merge(&self.values, same_change)
     }
 
     /// Pads this merge with to the specified number of sides with the specified
@@ -534,32 +577,6 @@ pub type MergedTreeVal<'a> = Merge<Option<&'a TreeValue>>;
 /// tree, it shouldn't be.
 pub type MergedTreeValue = Merge<Option<TreeValue>>;
 
-impl MergedTreeValue {
-    /// Create a `Merge` from a `backend::Conflict`, padding with `None` to
-    /// make sure that there is exactly one more `adds()` than `removes()`.
-    pub fn from_backend_conflict(conflict: backend::Conflict) -> Self {
-        let removes = conflict.removes.into_iter().map(|term| term.value);
-        let adds = conflict.adds.into_iter().map(|term| term.value);
-        Merge::from_legacy_form(removes, adds)
-    }
-
-    /// Creates a `backend::Conflict` from a `Merge` by dropping `None`
-    /// values. Note that the conversion is lossy: the order of `None` values is
-    /// not preserved when converting back to a `Merge`.
-    pub fn into_backend_conflict(self) -> backend::Conflict {
-        let (removes, adds) = self.into_legacy_form();
-        let removes = removes
-            .into_iter()
-            .map(|value| backend::ConflictTerm { value })
-            .collect();
-        let adds = adds
-            .into_iter()
-            .map(|value| backend::ConflictTerm { value })
-            .collect();
-        backend::Conflict { removes, adds }
-    }
-}
-
 impl<T> Merge<Option<T>>
 where
     T: Borrow<TreeValue>,
@@ -736,80 +753,82 @@ fn describe_conflict_term(value: &TreeValue) -> String {
         TreeValue::GitSubmodule(id) => {
             format!("Git submodule with id {id}")
         }
-        TreeValue::Conflict(id) => {
-            format!("Conflict with id {id}")
-        }
     }
 }
 
 #[cfg(test)]
 mod tests {
+    use test_case::test_case;
+
     use super::*;
 
     fn c<T: Clone>(terms: &[T]) -> Merge<T> {
         Merge::from_vec(terms.to_vec())
     }
 
-    #[test]
-    fn test_trivial_merge() {
-        assert_eq!(trivial_merge(&[0]), Some(&0));
-        assert_eq!(trivial_merge(&[0, 0, 0]), Some(&0));
-        assert_eq!(trivial_merge(&[0, 0, 1]), Some(&1));
-        assert_eq!(trivial_merge(&[0, 1, 0]), Some(&0));
-        assert_eq!(trivial_merge(&[0, 1, 1]), Some(&0));
-        assert_eq!(trivial_merge(&[0, 1, 2]), None);
-        assert_eq!(trivial_merge(&[0, 0, 0, 0, 0]), Some(&0));
-        assert_eq!(trivial_merge(&[0, 0, 0, 0, 1]), Some(&1));
-        assert_eq!(trivial_merge(&[0, 0, 0, 1, 0]), Some(&0));
-        assert_eq!(trivial_merge(&[0, 0, 0, 1, 1]), Some(&0));
-        assert_eq!(trivial_merge(&[0, 0, 0, 1, 2]), None);
-        assert_eq!(trivial_merge(&[0, 0, 1, 0, 0]), Some(&1));
-        assert_eq!(trivial_merge(&[0, 0, 1, 0, 1]), Some(&1));
-        assert_eq!(trivial_merge(&[0, 0, 1, 0, 2]), None);
-        assert_eq!(trivial_merge(&[0, 0, 1, 1, 0]), Some(&0));
-        assert_eq!(trivial_merge(&[0, 0, 1, 1, 1]), Some(&1));
-        assert_eq!(trivial_merge(&[0, 0, 1, 1, 2]), Some(&2));
-        assert_eq!(trivial_merge(&[0, 0, 1, 2, 0]), None);
-        assert_eq!(trivial_merge(&[0, 0, 1, 2, 1]), Some(&1));
-        assert_eq!(trivial_merge(&[0, 0, 1, 2, 2]), Some(&1));
-        assert_eq!(trivial_merge(&[0, 0, 1, 2, 3]), None);
-        assert_eq!(trivial_merge(&[0, 1, 0, 0, 0]), Some(&0));
-        assert_eq!(trivial_merge(&[0, 1, 0, 0, 1]), Some(&0));
-        assert_eq!(trivial_merge(&[0, 1, 0, 0, 2]), None);
-        assert_eq!(trivial_merge(&[0, 1, 0, 1, 0]), Some(&0));
-        assert_eq!(trivial_merge(&[0, 1, 0, 1, 1]), Some(&0));
-        assert_eq!(trivial_merge(&[0, 1, 0, 1, 2]), None);
-        assert_eq!(trivial_merge(&[0, 1, 0, 2, 0]), None);
-        assert_eq!(trivial_merge(&[0, 1, 0, 2, 1]), Some(&0));
-        assert_eq!(trivial_merge(&[0, 1, 0, 2, 2]), Some(&0));
-        assert_eq!(trivial_merge(&[0, 1, 0, 2, 3]), None);
-        assert_eq!(trivial_merge(&[0, 1, 1, 0, 0]), Some(&0));
-        assert_eq!(trivial_merge(&[0, 1, 1, 0, 1]), Some(&1));
-        assert_eq!(trivial_merge(&[0, 1, 1, 0, 2]), Some(&2));
-        assert_eq!(trivial_merge(&[0, 1, 1, 1, 0]), Some(&0));
-        assert_eq!(trivial_merge(&[0, 1, 1, 1, 1]), Some(&0));
-        assert_eq!(trivial_merge(&[0, 1, 1, 1, 2]), None);
-        assert_eq!(trivial_merge(&[0, 1, 1, 2, 0]), Some(&0));
-        assert_eq!(trivial_merge(&[0, 1, 1, 2, 1]), None);
-        assert_eq!(trivial_merge(&[0, 1, 1, 2, 2]), Some(&0));
-        assert_eq!(trivial_merge(&[0, 1, 1, 2, 3]), None);
-        assert_eq!(trivial_merge(&[0, 1, 2, 0, 0]), None);
-        assert_eq!(trivial_merge(&[0, 1, 2, 0, 1]), Some(&2));
-        assert_eq!(trivial_merge(&[0, 1, 2, 0, 2]), Some(&2));
-        assert_eq!(trivial_merge(&[0, 1, 2, 0, 3]), None);
-        assert_eq!(trivial_merge(&[0, 1, 2, 1, 0]), None);
-        assert_eq!(trivial_merge(&[0, 1, 2, 1, 1]), None);
-        assert_eq!(trivial_merge(&[0, 1, 2, 1, 2]), None);
-        assert_eq!(trivial_merge(&[0, 1, 2, 1, 3]), None);
-        assert_eq!(trivial_merge(&[0, 1, 2, 2, 0]), Some(&0));
-        assert_eq!(trivial_merge(&[0, 1, 2, 2, 1]), Some(&0));
-        assert_eq!(trivial_merge(&[0, 1, 2, 2, 2]), None);
-        assert_eq!(trivial_merge(&[0, 1, 2, 2, 3]), None);
-        assert_eq!(trivial_merge(&[0, 1, 2, 3, 0]), None);
-        assert_eq!(trivial_merge(&[0, 1, 2, 3, 1]), None);
-        assert_eq!(trivial_merge(&[0, 1, 2, 3, 2]), None);
-        assert_eq!(trivial_merge(&[0, 1, 2, 3, 3]), None);
-        assert_eq!(trivial_merge(&[0, 1, 2, 3, 4]), None);
+    #[test_case(SameChange::Keep)]
+    #[test_case(SameChange::Accept)]
+    fn test_trivial_merge(same_change: SameChange) {
+        let accept_same_change = same_change == SameChange::Accept;
+        let merge = |values| trivial_merge(values, same_change);
+        assert_eq!(merge(&[0]), Some(&0));
+        assert_eq!(merge(&[0, 0, 0]), Some(&0));
+        assert_eq!(merge(&[0, 0, 1]), Some(&1));
+        assert_eq!(merge(&[0, 1, 0]), accept_same_change.then_some(&0));
+        assert_eq!(merge(&[0, 1, 1]), Some(&0));
+        assert_eq!(merge(&[0, 1, 2]), None);
+        assert_eq!(merge(&[0, 0, 0, 0, 0]), Some(&0));
+        assert_eq!(merge(&[0, 0, 0, 0, 1]), Some(&1));
+        assert_eq!(merge(&[0, 0, 0, 1, 0]), accept_same_change.then_some(&0));
+        assert_eq!(merge(&[0, 0, 0, 1, 1]), Some(&0));
+        assert_eq!(merge(&[0, 0, 0, 1, 2]), None);
+        assert_eq!(merge(&[0, 0, 1, 0, 0]), Some(&1));
+        assert_eq!(merge(&[0, 0, 1, 0, 1]), accept_same_change.then_some(&1));
+        assert_eq!(merge(&[0, 0, 1, 0, 2]), None);
+        assert_eq!(merge(&[0, 0, 1, 1, 0]), Some(&0));
+        assert_eq!(merge(&[0, 0, 1, 1, 1]), Some(&1));
+        assert_eq!(merge(&[0, 0, 1, 1, 2]), Some(&2));
+        assert_eq!(merge(&[0, 0, 1, 2, 0]), None);
+        assert_eq!(merge(&[0, 0, 1, 2, 1]), accept_same_change.then_some(&1));
+        assert_eq!(merge(&[0, 0, 1, 2, 2]), Some(&1));
+        assert_eq!(merge(&[0, 0, 1, 2, 3]), None);
+        assert_eq!(merge(&[0, 1, 0, 0, 0]), accept_same_change.then_some(&0));
+        assert_eq!(merge(&[0, 1, 0, 0, 1]), Some(&0));
+        assert_eq!(merge(&[0, 1, 0, 0, 2]), None);
+        assert_eq!(merge(&[0, 1, 0, 1, 0]), accept_same_change.then_some(&0));
+        assert_eq!(merge(&[0, 1, 0, 1, 1]), accept_same_change.then_some(&0));
+        assert_eq!(merge(&[0, 1, 0, 1, 2]), None);
+        assert_eq!(merge(&[0, 1, 0, 2, 0]), None);
+        assert_eq!(merge(&[0, 1, 0, 2, 1]), accept_same_change.then_some(&0));
+        assert_eq!(merge(&[0, 1, 0, 2, 2]), accept_same_change.then_some(&0));
+        assert_eq!(merge(&[0, 1, 0, 2, 3]), None);
+        assert_eq!(merge(&[0, 1, 1, 0, 0]), Some(&0));
+        assert_eq!(merge(&[0, 1, 1, 0, 1]), Some(&1));
+        assert_eq!(merge(&[0, 1, 1, 0, 2]), Some(&2));
+        assert_eq!(merge(&[0, 1, 1, 1, 0]), accept_same_change.then_some(&0));
+        assert_eq!(merge(&[0, 1, 1, 1, 1]), Some(&0));
+        assert_eq!(merge(&[0, 1, 1, 1, 2]), None);
+        assert_eq!(merge(&[0, 1, 1, 2, 0]), accept_same_change.then_some(&0));
+        assert_eq!(merge(&[0, 1, 1, 2, 1]), None);
+        assert_eq!(merge(&[0, 1, 1, 2, 2]), Some(&0));
+        assert_eq!(merge(&[0, 1, 1, 2, 3]), None);
+        assert_eq!(merge(&[0, 1, 2, 0, 0]), None);
+        assert_eq!(merge(&[0, 1, 2, 0, 1]), Some(&2));
+        assert_eq!(merge(&[0, 1, 2, 0, 2]), accept_same_change.then_some(&2));
+        assert_eq!(merge(&[0, 1, 2, 0, 3]), None);
+        assert_eq!(merge(&[0, 1, 2, 1, 0]), None);
+        assert_eq!(merge(&[0, 1, 2, 1, 1]), None);
+        assert_eq!(merge(&[0, 1, 2, 1, 2]), None);
+        assert_eq!(merge(&[0, 1, 2, 1, 3]), None);
+        assert_eq!(merge(&[0, 1, 2, 2, 0]), accept_same_change.then_some(&0));
+        assert_eq!(merge(&[0, 1, 2, 2, 1]), Some(&0));
+        assert_eq!(merge(&[0, 1, 2, 2, 2]), None);
+        assert_eq!(merge(&[0, 1, 2, 2, 3]), None);
+        assert_eq!(merge(&[0, 1, 2, 3, 0]), None);
+        assert_eq!(merge(&[0, 1, 2, 3, 1]), None);
+        assert_eq!(merge(&[0, 1, 2, 3, 2]), None);
+        assert_eq!(merge(&[0, 1, 2, 3, 3]), None);
+        assert_eq!(merge(&[0, 1, 2, 3, 4]), None);
     }
 
     #[test]
@@ -1081,8 +1100,8 @@ mod tests {
             );
             // `resolve_trivial()` is unaffected by `simplify()`
             assert_eq!(
-                merge.simplify().resolve_trivial(),
-                merge.resolve_trivial(),
+                merge.simplify().resolve_trivial(SameChange::Accept),
+                merge.resolve_trivial(SameChange::Accept),
                 "simplify() changed result of resolve_trivial() for {merge:?}"
             );
         }
