@@ -12,7 +12,7 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
-#![allow(missing_docs)]
+#![expect(missing_docs)]
 
 use std::path::PathBuf;
 use std::str::FromStr;
@@ -35,7 +35,6 @@ use crate::config::ConfigValue;
 use crate::config::StackedConfig;
 use crate::config::ToConfigNamePath;
 use crate::fmt_util::binary_prefix;
-use crate::fsmonitor::FsmonitorSettings;
 use crate::signing::SignBehavior;
 
 #[derive(Debug, Clone)]
@@ -78,18 +77,6 @@ impl GitSettings {
     }
 }
 
-impl Default for GitSettings {
-    fn default() -> Self {
-        Self {
-            auto_local_bookmark: false,
-            abandon_unreachable_commits: true,
-            executable_path: PathBuf::from("git"),
-            write_change_id_header: true,
-            colocate: false,
-        }
-    }
-}
-
 /// Commit signing settings, describes how to and if to sign commits.
 #[derive(Debug, Clone)]
 pub struct SignSettings {
@@ -191,10 +178,6 @@ impl UserSettings {
         &self.data.user_email
     }
 
-    pub fn fsmonitor_settings(&self) -> Result<FsmonitorSettings, ConfigGetError> {
-        FsmonitorSettings::from_settings(self)
-    }
-
     // Must not be changed to avoid git pushing older commits with no set email
     // address
     pub const USER_EMAIL_PLACEHOLDER: &str = "(no email configured)";
