@@ -12,9 +12,8 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
-#![allow(missing_docs)]
+#![expect(missing_docs)]
 
-use std::any::Any;
 use std::fmt::Debug;
 use std::fs;
 use std::fs::File;
@@ -43,9 +42,6 @@ use crate::backend::BackendResult;
 use crate::backend::ChangeId;
 use crate::backend::Commit;
 use crate::backend::CommitId;
-use crate::backend::Conflict;
-use crate::backend::ConflictId;
-use crate::backend::ConflictTerm;
 use crate::backend::CopyHistory;
 use crate::backend::CopyId;
 use crate::backend::CopyRecord;
@@ -150,18 +146,10 @@ impl SimpleBackend {
     fn commit_path(&self, id: &CommitId) -> PathBuf {
         self.path.join("commits").join(id.hex())
     }
-
-    fn conflict_path(&self, id: &ConflictId) -> PathBuf {
-        self.path.join("conflicts").join(id.hex())
-    }
 }
 
 #[async_trait]
 impl Backend for SimpleBackend {
-    fn as_any(&self) -> &dyn Any {
-        self
-    }
-
     fn name(&self) -> &str {
         Self::name()
     }
@@ -298,31 +286,6 @@ impl Backend for SimpleBackend {
         Ok(id)
     }
 
-    fn read_conflict(&self, _path: &RepoPath, id: &ConflictId) -> BackendResult<Conflict> {
-        let path = self.conflict_path(id);
-        let buf = fs::read(path).map_err(|err| map_not_found_err(err, id))?;
-
-        let proto = crate::protos::simple_store::Conflict::decode(&*buf).map_err(to_other_err)?;
-        Ok(conflict_from_proto(proto))
-    }
-
-    fn write_conflict(&self, _path: &RepoPath, conflict: &Conflict) -> BackendResult<ConflictId> {
-        // TODO: Write temporary file in the destination directory (#5712)
-        let temp_file = NamedTempFile::new_in(&self.path).map_err(to_other_err)?;
-
-        let proto = conflict_to_proto(conflict);
-        temp_file
-            .as_file()
-            .write_all(&proto.encode_to_vec())
-            .map_err(to_other_err)?;
-
-        let id = ConflictId::new(blake2b_hash(conflict).to_vec());
-
-        persist_content_addressed_temp_file(temp_file, self.conflict_path(&id))
-            .map_err(to_other_err)?;
-        Ok(id)
-    }
-
     async fn read_commit(&self, id: &CommitId) -> BackendResult<Commit> {
         if *id == self.root_commit_id {
             return Ok(make_root_commit(
@@ -387,7 +350,7 @@ impl Backend for SimpleBackend {
     }
 }
 
-#[allow(clippy::assigning_clones)]
+#[expect(clippy::assigning_clones)]
 pub fn commit_to_proto(commit: &Commit) -> crate::protos::simple_store::Commit {
     let mut proto = crate::protos::simple_store::Commit::default();
     for parent in &commit.parents {
@@ -397,11 +360,10 @@ pub fn commit_to_proto(commit: &Commit) -> crate::protos::simple_store::Commit {
         proto.predecessors.push(predecessor.to_bytes());
     }
     match &commit.root_tree {
-        MergedTreeId::Legacy(tree_id) => {
-            proto.root_tree = vec![tree_id.to_bytes()];
+        MergedTreeId::Legacy(_) => {
+            panic!("The simple backend doesn't support legacy trees");
         }
         MergedTreeId::Merge(tree_ids) => {
-            proto.uses_tree_conflict_format = true;
             proto.root_tree = tree_ids.iter().map(|id| id.to_bytes()).collect();
         }
     }
@@ -422,13 +384,8 @@ fn commit_from_proto(mut proto: crate::protos::simple_store::Commit) -> Commit {
 
     let parents = proto.parents.into_iter().map(CommitId::new).collect();
     let predecessors = proto.predecessors.into_iter().map(CommitId::new).collect();
-    let root_tree = if proto.uses_tree_conflict_format {
-        let merge_builder: MergeBuilder<_> = proto.root_tree.into_iter().map(TreeId::new).collect();
-        MergedTreeId::Merge(merge_builder.build())
-    } else {
-        assert_eq!(proto.root_tree.len(), 1);
-        MergedTreeId::Legacy(TreeId::new(proto.root_tree[0].clone()))
-    };
+    let merge_builder: MergeBuilder<_> = proto.root_tree.into_iter().map(TreeId::new).collect();
+    let root_tree = MergedTreeId::Merge(merge_builder.build());
     let change_id = ChangeId::new(proto.change_id);
     Commit {
         parents,
@@ -497,11 +454,6 @@ fn tree_value_to_proto(value: &TreeValue) -> crate::protos::simple_store::TreeVa
                 id.to_bytes(),
             ));
         }
-        TreeValue::Conflict(id) => {
-            proto.value = Some(crate::protos::simple_store::tree_value::Value::ConflictId(
-                id.to_bytes(),
-            ));
-        }
     }
     proto
 }
@@ -525,9 +477,6 @@ fn tree_value_from_proto(proto: crate::protos::simple_store::TreeValue) -> TreeV
         crate::protos::simple_store::tree_value::Value::SymlinkId(id) => {
             TreeValue::Symlink(SymlinkId::new(id))
         }
-        crate::protos::simple_store::tree_value::Value::ConflictId(id) => {
-            TreeValue::Conflict(ConflictId::new(id))
-        }
     }
 }
 
@@ -554,43 +503,6 @@ fn signature_from_proto(proto: crate::protos::simple_store::commit::Signature) -
     }
 }
 
-fn conflict_to_proto(conflict: &Conflict) -> crate::protos::simple_store::Conflict {
-    let mut proto = crate::protos::simple_store::Conflict::default();
-    for term in &conflict.removes {
-        proto.removes.push(conflict_term_to_proto(term));
-    }
-    for term in &conflict.adds {
-        proto.adds.push(conflict_term_to_proto(term));
-    }
-    proto
-}
-
-fn conflict_from_proto(proto: crate::protos::simple_store::Conflict) -> Conflict {
-    let removes = proto
-        .removes
-        .into_iter()
-        .map(conflict_term_from_proto)
-        .collect();
-    let adds = proto
-        .adds
-        .into_iter()
-        .map(conflict_term_from_proto)
-        .collect();
-    Conflict { removes, adds }
-}
-
-fn conflict_term_from_proto(proto: crate::protos::simple_store::conflict::Term) -> ConflictTerm {
-    ConflictTerm {
-        value: tree_value_from_proto(proto.content.unwrap()),
-    }
-}
-
-fn conflict_term_to_proto(part: &ConflictTerm) -> crate::protos::simple_store::conflict::Term {
-    crate::protos::simple_store::conflict::Term {
-        content: Some(tree_value_to_proto(&part.value)),
-    }
-}
-
 #[cfg(test)]
 mod tests {
     use assert_matches::assert_matches;
