@@ -12,9 +12,8 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
-#![allow(missing_docs)]
+#![expect(missing_docs)]
 
-use std::any::Any;
 use std::fmt::Debug;
 use std::fmt::Formatter;
 use std::pin::Pin;
@@ -32,7 +31,6 @@ use crate::backend::Backend;
 use crate::backend::BackendResult;
 use crate::backend::ChangeId;
 use crate::backend::CommitId;
-use crate::backend::ConflictId;
 use crate::backend::CopyRecord;
 use crate::backend::FileId;
 use crate::backend::MergedTreeId;
@@ -41,13 +39,12 @@ use crate::backend::SymlinkId;
 use crate::backend::TreeId;
 use crate::commit::Commit;
 use crate::index::Index;
-use crate::merge::Merge;
-use crate::merge::MergedTreeValue;
 use crate::merged_tree::MergedTree;
 use crate::repo_path::RepoPath;
 use crate::repo_path::RepoPathBuf;
 use crate::signing::Signer;
 use crate::tree::Tree;
+use crate::tree_merge::MergeOptions;
 
 // There are more tree objects than commits, and trees are often shared across
 // commits.
@@ -61,6 +58,7 @@ pub struct Store {
     signer: Signer,
     commit_cache: Mutex<CLruCache<CommitId, Arc<backend::Commit>>>,
     tree_cache: Mutex<CLruCache<(RepoPathBuf, TreeId), Arc<backend::Tree>>>,
+    merge_options: MergeOptions,
 }
 
 impl Debug for Store {
@@ -72,12 +70,17 @@ impl Debug for Store {
 }
 
 impl Store {
-    pub fn new(backend: Box<dyn Backend>, signer: Signer) -> Arc<Self> {
+    pub fn new(
+        backend: Box<dyn Backend>,
+        signer: Signer,
+        merge_options: MergeOptions,
+    ) -> Arc<Self> {
         Arc::new(Self {
             backend,
             signer,
             commit_cache: Mutex::new(CLruCache::new(COMMIT_CACHE_CAPACITY.try_into().unwrap())),
             tree_cache: Mutex::new(CLruCache::new(TREE_CACHE_CAPACITY.try_into().unwrap())),
+            merge_options,
         })
     }
 
@@ -85,14 +88,20 @@ impl Store {
         self.backend.as_ref()
     }
 
-    pub fn backend_impl(&self) -> &dyn Any {
-        self.backend.as_any()
+    /// Returns backend as the implementation type.
+    pub fn backend_impl<T: Backend>(&self) -> Option<&T> {
+        self.backend.downcast_ref()
     }
 
     pub fn signer(&self) -> &Signer {
         &self.signer
     }
 
+    /// Default merge options to be used when resolving parent trees.
+    pub fn merge_options(&self) -> &MergeOptions {
+        &self.merge_options
+    }
+
     pub fn get_copy_records(
         &self,
         paths: Option<&[RepoPathBuf]>,
@@ -217,7 +226,7 @@ impl Store {
         match &id {
             MergedTreeId::Legacy(id) => {
                 let tree = self.get_tree_async(RepoPathBuf::root(), id).await?;
-                MergedTree::from_legacy_tree(tree)
+                Ok(MergedTree::resolved(tree))
             }
             MergedTreeId::Merge(ids) => {
                 let trees = ids
@@ -267,24 +276,6 @@ impl Store {
         self.backend.write_symlink(path, contents).await
     }
 
-    pub fn read_conflict(
-        &self,
-        path: &RepoPath,
-        id: &ConflictId,
-    ) -> BackendResult<MergedTreeValue> {
-        let backend_conflict = self.backend.read_conflict(path, id)?;
-        Ok(Merge::from_backend_conflict(backend_conflict))
-    }
-
-    pub fn write_conflict(
-        &self,
-        path: &RepoPath,
-        contents: &MergedTreeValue,
-    ) -> BackendResult<ConflictId> {
-        self.backend
-            .write_conflict(path, &contents.clone().into_backend_conflict())
-    }
-
     pub fn gc(&self, index: &dyn Index, keep_newer: SystemTime) -> BackendResult<()> {
         self.backend.gc(index, keep_newer)
     }
