@@ -12,7 +12,7 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
-#![allow(missing_docs)]
+#![expect(missing_docs)]
 
 use std::collections::HashMap;
 use std::fs;
@@ -435,10 +435,10 @@ impl Workspace {
         // the caller expected. It's safe to check out another commit
         // regardless, but it's probably not what  the caller wanted, so we let
         // them know.
-        if let Some(old_tree_id) = old_tree_id {
-            if old_tree_id != locked_ws.locked_wc().old_tree_id() {
-                return Err(CheckoutError::ConcurrentCheckout);
-            }
+        if let Some(old_tree_id) = old_tree_id
+            && old_tree_id != locked_ws.locked_wc().old_tree_id()
+        {
+            return Err(CheckoutError::ConcurrentCheckout);
         }
         let stats = locked_ws.locked_wc().check_out(commit)?;
         locked_ws
