@@ -15,14 +15,14 @@
 use std::fs;
 #[cfg(unix)]
 use std::fs::Permissions;
-use std::io::Write;
+use std::io::Write as _;
 #[cfg(unix)]
-use std::os::unix::prelude::PermissionsExt;
+use std::os::unix::prelude::PermissionsExt as _;
 use std::path::Path;
 use std::path::PathBuf;
 
 use jj_lib::signing::SigStatus;
-use jj_lib::signing::SigningBackend;
+use jj_lib::signing::SigningBackend as _;
 use jj_lib::ssh_signing::SshBackend;
 
 static PRIVATE_KEY: &str = r#"-----BEGIN OPENSSH PRIVATE KEY-----
@@ -41,6 +41,7 @@ struct SshEnvironment {
     _keys: tempfile::TempDir,
     private_key_path: PathBuf,
     allowed_signers: Option<tempfile::TempPath>,
+    revocation_list: Option<tempfile::TempPath>,
 }
 
 impl SshEnvironment {
@@ -55,12 +56,13 @@ impl SshEnvironment {
         fs::write(&private_key_path, PRIVATE_KEY).unwrap();
 
         #[cfg(unix)]
-        std::fs::set_permissions(&private_key_path, Permissions::from_mode(0o700)).unwrap();
+        std::fs::set_permissions(&private_key_path, Permissions::from_mode(0o600)).unwrap();
 
-        let mut env = SshEnvironment {
+        let mut env = Self {
             _keys: keys_dir,
             private_key_path,
             allowed_signers: None,
+            revocation_list: None,
         };
 
         env.with_good_public_key();
@@ -103,6 +105,20 @@ impl SshEnvironment {
 
         self.allowed_signers = Some(allowed_signers_path);
     }
+
+    fn with_revocation_list(&mut self, revoked_key: &[u8]) {
+        let mut revocation_list = tempfile::Builder::new()
+            .prefix("jj-test-revocation-list-")
+            .tempfile()
+            .unwrap();
+
+        revocation_list.write_all(revoked_key).unwrap();
+        revocation_list.flush().unwrap();
+
+        let revocation_list_path = revocation_list.into_temp_path();
+
+        self.revocation_list = Some(revocation_list_path);
+    }
 }
 
 fn backend(env: &SshEnvironment) -> SshBackend {
@@ -111,6 +127,9 @@ fn backend(env: &SshEnvironment) -> SshBackend {
         env.allowed_signers
             .as_ref()
             .map(|allowed_signers| allowed_signers.as_os_str().into()),
+        env.revocation_list
+            .as_ref()
+            .map(|revocation_list| revocation_list.as_os_str().into()),
     )
 }
 
@@ -167,3 +186,38 @@ fn ssh_signing_missing_allowed_signers() {
     assert_eq!(check.status, SigStatus::Unknown);
     assert_eq!(check.display.unwrap(), "Signature OK. Unknown principal");
 }
+
+#[test]
+fn ssh_signing_revocation_revoked() {
+    let mut env = SshEnvironment::new().unwrap();
+    env.with_revocation_list(PUBLIC_KEY.as_bytes());
+
+    let backend = backend(&env);
+    let data = b"hello world";
+
+    let signature = backend
+        .sign(data, Some(env.private_key_path.to_str().unwrap()))
+        .unwrap();
+
+    let check = backend.verify(data, &signature).unwrap();
+    assert_eq!(check.status, SigStatus::Bad);
+    assert_eq!(check.display.unwrap(), "test@example.com");
+}
+
+#[test]
+fn ssh_signing_revocation_unrevoked() {
+    let mut env = SshEnvironment::new().unwrap();
+    const ALT_PUB_KEY: &str =
+        "ssh-ed25519 AAAAC3NzaC1lZDI1NTE5AAAAICrkiOk+QyRv87ahGdrxSin0DuNKezDDLE6lLkHxJpWU";
+    env.with_revocation_list(ALT_PUB_KEY.as_bytes());
+
+    let backend = backend(&env);
+    let data = b"hello world";
+
+    let signature = backend
+        .sign(data, Some(env.private_key_path.to_str().unwrap()))
+        .unwrap();
+
+    let check = backend.verify(data, &signature).unwrap();
+    assert_eq!(check.status, SigStatus::Good);
+}
