@@ -12,7 +12,6 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
-use std::any::Any;
 use std::collections::HashMap;
 use std::fmt::Debug;
 use std::fmt::Error;
@@ -35,8 +34,6 @@ use jj_lib::backend::BackendResult;
 use jj_lib::backend::ChangeId;
 use jj_lib::backend::Commit;
 use jj_lib::backend::CommitId;
-use jj_lib::backend::Conflict;
-use jj_lib::backend::ConflictId;
 use jj_lib::backend::CopyHistory;
 use jj_lib::backend::CopyId;
 use jj_lib::backend::CopyRecord;
@@ -71,7 +68,6 @@ pub struct TestBackendData {
     trees: HashMap<RepoPathBuf, HashMap<TreeId, Tree>>,
     files: HashMap<RepoPathBuf, HashMap<FileId, Vec<u8>>>,
     symlinks: HashMap<RepoPathBuf, HashMap<SymlinkId, String>>,
-    conflicts: HashMap<RepoPathBuf, HashMap<ConflictId, Conflict>>,
     copies: HashMap<CopyId, CopyHistory>,
 }
 
@@ -172,10 +168,6 @@ impl Debug for TestBackend {
 
 #[async_trait]
 impl Backend for TestBackend {
-    fn as_any(&self) -> &dyn Any {
-        self
-    }
-
     fn name(&self) -> &str {
         "test"
     }
@@ -378,33 +370,6 @@ impl Backend for TestBackend {
         .await
     }
 
-    fn read_conflict(&self, path: &RepoPath, id: &ConflictId) -> BackendResult<Conflict> {
-        match self
-            .locked_data()
-            .conflicts
-            .get(path)
-            .and_then(|items| items.get(id))
-            .cloned()
-        {
-            None => Err(BackendError::ObjectNotFound {
-                object_type: "conflict".to_string(),
-                hash: id.hex(),
-                source: format!("at path {path:?}").into(),
-            }),
-            Some(conflict) => Ok(conflict),
-        }
-    }
-
-    fn write_conflict(&self, path: &RepoPath, contents: &Conflict) -> BackendResult<ConflictId> {
-        let id = ConflictId::new(get_hash(contents));
-        self.locked_data()
-            .conflicts
-            .entry(path.to_owned())
-            .or_default()
-            .insert(id.clone(), contents.clone());
-        Ok(id)
-    }
-
     async fn read_commit(&self, id: &CommitId) -> BackendResult<Commit> {
         if id == &self.root_commit_id {
             return Ok(make_root_commit(
